import os

import numpy as np
import pandas as pd
import scipy
from gensim.matutils import Sparse2Corpus
from penelope.utility import flatten

PUBLICATION2ID = {'AFTONBLADET': 1, 'EXPRESSEN': 2, 'DAGENS NYHETER': 3, 'SVENSKA DAGBLADET': 4}
ID2PUBLICATION = {v: k for k, v in PUBLICATION2ID.items()}
ID2PUB = {1: 'AB', 2: 'EX', 3: 'DN', 4: 'SVD'}
PUB2ID = {v: k for k, v in ID2PUB.items()}

corpus_dataset_filename = "corpus_dataset.zip"
document_dataset_filename = "document_dataset.zip"
vocabulary_dataset_filename = "vocabulary_dataset.zip"
censored_corpus_filename = "text1_utantext.zip"
meta_textblocks_filename = "meta_textblocks.zip"
sparse_matrix_filename = "corpus_sparse_doc_term_matrx.npz"
document_processed_filename = "document_processed_dataset.zip"
reconstructed_text_corpus_file = "reconstructed_text_corpus.csv.zip"


def load_corpus_dtm_as_data_frame(corpus_folder):
    """Load corpus DTM, source "dtm1.rds", arrays drm$i, drm$j, drm$v'"""

    filename = os.path.join(corpus_folder, corpus_dataset_filename)

    df_corpus = pd.read_csv(filename, compression='zip', header=0, sep=',', quotechar='"', na_filter=False)
    df_corpus.columns = ["document_id", "token_id", "tf"]
    df_corpus.document_id -= 1
    df_corpus.token_id -= 1

    return df_corpus


def load_vocabulary_file_as_data_frame(corpus_folder):
    """Load vocabulary"""

    filename = os.path.join(corpus_folder, vocabulary_dataset_filename)

    df_vocabulary = pd.read_csv(filename, compression='zip', header=0, sep=',', quotechar='"', na_filter=False)
    df_vocabulary.columns = ["token"]

    return df_vocabulary


def load_censured_text_as_data_frame(corpus_folder):
    """ Load censored corpus data """

    filename = os.path.join(corpus_folder, censored_corpus_filename)

    df_censured_text = pd.read_csv(filename, compression='zip', header=0, sep=',', quotechar='"', na_filter=False)
    df_censured_text.columns = ['id', 'doc_id', 'publication', 'date']
    df_censured_text.id -= 1

    df_censured_text = df_censured_text[['doc_id', 'publication', 'date']].drop_duplicates()

    return df_censured_text


def load_meta_text_blocks_as_data_frame(corpus_folder):
    """ Load censored corpus data """

    filename = os.path.join(corpus_folder, meta_textblocks_filename)
    df_meta = pd.read_csv(filename, compression='zip', header=0, sep=',', quotechar='"', na_filter=False)
    df_meta = df_meta[['id', 'pred_bodytext']].drop_duplicates()
    df_meta.columns = ["doc_id", "pred_bodytext"]
    df_meta = df_meta.set_index("doc_id")
    return df_meta


def load_documents(corpus_folder, force=False):
    """ Load documents data, source "dtm1.rds", arrays drm$dimnames[1] """

    processed_filename = os.path.join(corpus_folder, document_processed_filename)

    if not os.path.isfile(processed_filename) or force:

        filename = os.path.join(corpus_folder, document_dataset_filename)

        df_document = pd.read_csv(filename, compression='zip', header=0, sep=',', quotechar='"', na_filter=False)
        df_document.columns = ["doc_id"]
        df_document.index.name = 'id'

        # Add publication and date
        df_censured_text = load_censured_text_as_data_frame(corpus_folder)
        df_document = pd.merge(
            df_document,
            df_censured_text[['doc_id', 'publication', 'date']],
            how='inner',
            left_on='doc_id',
            right_on='doc_id',
        )

        # Add pred_bodytext
        df_meta = load_meta_text_blocks_as_data_frame(corpus_folder)
        df_document = pd.merge(df_document, df_meta, how='inner', left_on='doc_id', right_index=True)

        # Add year
        df_document['date'] = pd.to_datetime(df_document.date)
        df_document['year'] = df_document.date.dt.year
        df_document['publication_id'] = df_document.publication.apply(lambda x: PUBLICATION2ID[x]).astype(np.uint16)

        df_document.to_csv(
            processed_filename, compression='zip', header=True, sep=',', quotechar='"', index=True, index_label="id"
        )

    else:
        # loading cached...
        df_document = pd.read_csv(
            processed_filename, compression='zip', header=0, sep=',', quotechar='"', na_filter=False, index_col="id"
        )
        if 'publication_id' not in df_document.columns:
            df_document['publication_id'] = df_document.publication.apply(lambda x: PUBLICATION2ID[x]).astype(np.uint16)

    return df_document


def load(corpus_folder):

    df_corpus = load_corpus_dtm_as_data_frame(corpus_folder)
    df_vocabulary = load_vocabulary_file_as_data_frame(corpus_folder)
    df_document = load_documents(corpus_folder)

    return df_corpus, df_document, df_vocabulary


def load_as_sparse_matrix(corpus_folder):

    filename = os.path.join(corpus_folder, sparse_matrix_filename)

    if not os.path.isfile(filename):
        df_corpus = load_corpus_dtm_as_data_frame(corpus_folder)
        v_dtm = scipy.sparse.coo_matrix((df_corpus.tf, (df_corpus.document_id, df_corpus.token_id)))
        scipy.sparse.save_npz(filename, v_dtm, compressed=True)
    else:
        v_dtm = scipy.sparse.load_npz(filename)

    return v_dtm


def load_reconstructed_text_corpus(corpus_folder):
    filename = os.path.join(corpus_folder, reconstructed_text_corpus_file)
    if not os.path.isfile(filename):
        df_corpus = load_corpus_dtm_as_data_frame(corpus_folder)
        df_vocabulary = load_vocabulary_file_as_data_frame(corpus_folder)
        id2token = df_vocabulary['token'].to_dict()
        df_reconstructed_text_corpus = (df_corpus.groupby('document_id')).apply(
            lambda x: ' '.join(flatten(x['tf'] * (x['token_id'].apply(lambda y: [id2token[y]]))))
        )
        # FIXME Is extra index written? Is headers written? Might be that first row is ignored???
        df_reconstructed_text_corpus.to_csv(filename, compression='zip', header=0, sep=',', quotechar='"')
    else:
        df_reconstructed_text_corpus = pd.read_csv(filename, compression='zip', header=None, sep=',', quotechar='"')
        df_reconstructed_text_corpus.columns = ['document_id', 'text']
        df_reconstructed_text_corpus = df_reconstructed_text_corpus.set_index('document_id')

    return df_reconstructed_text_corpus


def load_as_dtm(corpus_folder):

    dtm = load_as_sparse_matrix(corpus_folder)
    id2token = load_vocabulary_file_as_data_frame(corpus_folder)['token'].to_dict()
    documents = load_documents(corpus_folder)

    return dtm, documents, id2token


def load_as_dtm2(corpus_folder, publication_ids=None):

    dtm, documents, id2token = load_as_dtm(corpus_folder)

    if publication_ids is not None:

        documents = documents[documents.publication_id.isin(publication_ids)]
        dtm = dtm.tocsr()[documents.index, :]
        documents = documents.reset_index().drop('id', axis=1)
        token_ids = dtm.sum(axis=0).nonzero()[1]
        dtm = dtm[:, token_ids]
        id2token = {i: id2token[k] for i, k in enumerate(token_ids)}

    return dtm, documents, id2token


def load_as_gensim_sparse_corpus(corpus_folder):
    dtm, documents, id2token = load_as_dtm(corpus_folder)
    corpus = Sparse2Corpus(dtm, documents_columns=False)
    documents['n_terms'] = np.asarray(corpus.sparse.sum(axis=0)).reshape(-1).astype(np.uint32)
    return corpus, documents, id2token


def load_as_gensim_sparse_corpus2(corpus_folder, publication_ids=None):

    dtm, documents, id2token = load_as_dtm(corpus_folder)

    if publication_ids is not None:
        documents = documents[documents.publication_id.isin(publication_ids)]
        dtm = dtm.tocsr()[documents.index, :]
        documents = documents.reset_index().drop('id', axis=1)
        token_ids = dtm.sum(axis=0).nonzero()[1]
        dtm = dtm[:, token_ids]
        id2token = {i: id2token[k] for i, k in enumerate(token_ids)}

    corpus = Sparse2Corpus(dtm, documents_columns=False)
    documents['n_terms'] = np.asarray(corpus.sparse.sum(axis=0)).reshape(-1).astype(np.uint32)
    return corpus, documents, id2token


def load_dates_subset_as_dtm(corpus_folder, dates):
    dtm, documents, id2token = load_as_dtm(corpus_folder)
    documents = documents[documents.date.isin(dates)]
    dtm = dtm.tocsr()[documents.index, :]
    documents = documents.reset_index().drop('id', axis=1)
    token_ids = dtm.sum(axis=0).nonzero()[1]
    dtm = dtm[:, token_ids]
    id2token = {i: id2token[k] for i, k in enumerate(token_ids)}
    return dtm, documents, id2token


def slim_documents(documents):

    # df = documents[['publication', 'year']].copy()
    # df['publication_id'] = df.publication.apply(lambda x: PUBLICATION2ID[x]).astype(np.uint16)
    # df = df[['publication_id', 'year']]
    return documents[['publication_id', 'year']].copy()


def extend_with_document_info(df, documents):
    """ Adds document meta data to given data frame (must have a document_id) """
    df = df.merge(documents, how='inner', left_on='document_id', right_index=True)
    return df



# ---
# jupyter:
#   jupytext:
#     formats: ipynb,py:percent
#     text_representation:
#       extension: .py
#       format_name: percent
#       format_version: '1.3'
#       jupytext_version: 1.6.0
#   kernelspec:
#     display_name: Python 3
#     language: python
#     name: python3
# ---

# %% [markdown]
# ## Word Significance using TF-IDF

# %%
# %load_ext autoreload
# %autoreload 2

# pylint: disable=wrong-import-position

import logging
import os
import sys

import ipywidgets
import numpy as np
import pandas as pd
import penelope.corpus.vectorized_corpus as vectorized_corpus
from IPython.display import display
from sklearn.feature_extraction.text import TfidfVectorizer

root_folder = os.path.abspath(os.path.join(globals()["_dh"][-1], "../.."))

corpus_folder = os.path.join(root_folder, "output")

sys.path = [root_folder] + sys.path


logger = logging.getLogger(__name__)


# %% [markdown]
# ## Load previously vectorized corpus
#
# The corpus was created using the following settings:
#  - Tokens were converted to lower case.
#  - Only tokens that contains at least one alphanumeric character (only_alphanumeric).
#  - Accents are ot removed (remove_accents)
#  - Min token length 2 (min_len)
#  - Max length not set (max_len)
#  - Numerals are removed (keep_numerals, -N)
#  - Symbols are removed (keep_symbols, -S)
#
# Use the `corpus_vectorizer` module to create a new corpus with different settings.
#
# The loaded corpus is processed in the following ways:
#
#  - Exclude tokens having a total word count less than 10
#  - Include at most 50000 most frequent words words.
#
# Compute a new TF-IDF weighted corpus
#
#  - Group documents by year
#  - Compute mean TF-IDF for each year
#
# Some references:
#
#  - [scikit-learn TfidfTransformer](https://scikit-learn.org/stable/modules/generated/sklearn.feature_extraction.text.TfidfTransformer.html#sklearn.feature_extraction.text.TfidfTransformer)
#  - [Spark MLlib TF-IDF](https://spark.apache.org/docs/2.2.0/ml-features.html#tf-idf)
#
#

# %%

# pylint: disable=wrong-import-position

v_corpus = (
    vectorized_corpus.VectorizedCorpus.load(tag="SOU_1945-1989_NN+VB+JJ_lemma_L0_+N_+S", folder=corpus_folder)
    .slice_by_n_count(10)
    .slice_by_n_top(500000)
)

tf_idf_corpus = v_corpus.tf_idf().group_by_year2(aggregate_function="mean", dtype=np.float)

# normalized_corpus = v_corpus\
#    .normalize()\
#    .group_by_year2(aggregate_function='mean', dtype=np.float)


# %%


def display_top_terms(data):

    if data is None:
        logger.info("No data to display!")
        return

    df = pd.DataFrame({k: [x[0] for x in v] for k, v in data.items()})

    display(df)


def compute_top_terms(x_corpus, n_top, idx_groups=None):

    data = {x["label"]: x_corpus.get_top_n_words(n=n_top, indices=x["indices"]) for x in idx_groups}
    return data


def display_gui(x_corpus, x_documents):

    lw = lambda w: ipywidgets.Layout(width=w)

    year_min, year_max = x_documents.year.min(), x_documents.year.max()
    years = list(range(year_min, year_max + 1))
    decades = [10 * decade for decade in range((year_min // 10), (year_max // 10) + 1)]
    lustrums = [lustrum for lustrum in range(year_min - year_min % 5, year_max - year_max % 5, 5)]

    groups = [
        (
            "year",
            [{"label": str(year), "indices": [year - year_min]} for year in years],
        ),
        (
            "decade",
            [
                {
                    "label": str(decade),
                    "indices": [year - year_min for year in range(decade, decade + 10) if year_min <= year <= year_max],
                }
                for decade in decades
            ],
        ),
        (
            "lustrum",
            [
                {
                    "label": str(lustrum),
                    "indices": [
                        year - year_min for year in range(lustrum, lustrum + 5) if year_min <= year <= year_max
                    ],
                }
                for lustrum in lustrums
            ],
        ),
    ]

    w_n_top = ipywidgets.IntSlider(
        description="#words",
        min=10,
        max=1000,
        value=100,
        tooltip="Number of words to compute",
    )
    w_compute = ipywidgets.Button(description="Compute", icon="", button_style="Success", layout=lw("120px"))
    w_output = ipywidgets.Output()  # layout={'border': '1px solid black'})
    w_groups = ipywidgets.Dropdown(options=groups, value=groups[0][1], description="Groups:")

    boxes = ipywidgets.VBox(
        [
            ipywidgets.HBox(
                [w_n_top, w_groups, w_compute],
                layout=ipywidgets.Layout(align_items="flex-end"),
            ),
            w_output,
        ]
    )

    display(boxes)

    def compute_callback_handler(*_):
        w_output.clear_output()
        with w_output:
            try:

                w_compute.disabled = True

                data = compute_top_terms(
                    x_corpus,
                    n_top=w_n_top.value,
                    idx_groups=w_groups.value,
                )

                display_top_terms(data)

            except Exception as ex:
                logger.error(ex)
            finally:
                w_compute.disabled = False

    w_compute.on_click(compute_callback_handler)


display_gui(tf_idf_corpus, tf_idf_corpus.documents)


# %% jupyter={"source_hidden": true}

# %matplotlib inline


def plot_word(x_corpus, word):
    wv = x_corpus.get_word_vector(word)

    df = pd.DataFrame({"count": wv, "year": x_corpus.documents.year}).set_index("year")
    df.plot()


plot_word(v_corpus, "arbete")
plot_word(tf_idf_corpus, "arbete")
# plot_word(yearly_tf_idf_corpus, "arbete")


# %% jupyter={"source_hidden": true}


# np.nansum(a, axis=None, dtype=None, out=None, keepdims=<no value>)[source]
# np.nanmean(a, axis=None, dtype=None, out=None, keepdims=<no value>)[source]
docs = [
    "the house had a tiny little mouse",
    "the cat saw the mouse",
    "the mouse ran away from the house",
    "the cat finally ate the mouse",
    "the end of the mouse story",
]

tfidf_vectorizer = TfidfVectorizer(use_idf=True)
tfidf_vectorizer_vectors = tfidf_vectorizer.fit_transform(docs)

tfidf = tfidf_vectorizer_vectors.todense()
# TFIDF of words not in the doc will be 0, so replace them with nan
tfidf[tfidf == 0] = np.nan
# Use nanmean of numpy which will ignore nan while calculating the mean
means = np.nansum(tfidf, axis=0)
# convert it into a dictionary for later lookup
means = dict(zip(tfidf_vectorizer.get_feature_names(), means.tolist()[0]))

tfidf = tfidf_vectorizer_vectors.todense()
# Argsort the full TFIDF dense vector
ordered = np.argsort(tfidf * -1)
words = tfidf_vectorizer.get_feature_names()

top_k = 5
for i, doc in enumerate(docs):
    result = {}
    # Pick top_k from each argsorted matrix for each doc
    for t in range(top_k):
        # Pick the top k word, find its average tfidf from the
        # precomputed dictionary using nanmean and save it to later use
        result[words[ordered[i, t]]] = means[words[ordered[i, t]]]
    print(result)



# ---
# jupyter:
#   jupytext:
#     formats: ipynb,py:percent
#     text_representation:
#       extension: .py
#       format_name: percent
#       format_version: '1.3'
#       jupytext_version: 1.6.0
#   kernelspec:
#     display_name: Python 3
#     language: python
#     name: python3
# ---

# %% [markdown]
# ## Most Discriminating Terms
# Note! The computation can take a very long time depending on corpus size and arguments. It might be preferable to use the CLI version instead.
#
# ### Example CLI call
# Compare AB 1949-1949 to DN 1989-1989. Only the 200 top words are included in the computation, and at most 100 words are returned per group. Words that occur in few documents ( less than 3%), and many documents (more than 95%) are filtered out.
#
# ```
# python mdw_runner.py --group AB 1945 1949 --group DN 1985 1989 --top-n-terms 200 --max-n-terms 100 --min-df 3.0 --max-df 95.0
# ```
#
# The result is stored in an excel file which is named based on the execution parameters.
#

# %%
# %load_ext autoreload
# %autoreload 2

# from beakerx import *
# from beakerx.object import beakerx

import os
import sys

from .notebook_gui import mdw_gui


def get_root_folder():
    parts = os.getcwd().split(os.path.sep)
    parts = parts[: parts.index("welfare_state_analytics") + 1]
    return os.path.join("/", *parts)


if os.environ.get("JUPYTER_IMAGE_SPEC", "") == "westac_lab":
    root_folder = "/home/jovyan/welfare_state_analytics"
    corpus_folder = "/data/westac/textblock_politisk"
else:
    root_folder = get_root_folder()
    corpus_folder = os.path.join(root_folder, "data/textblock_politisk")

sys.path = [root_folder] + sys.path


v_corpus = mdw_gui.load_vectorized_corpus(corpus_folder, [1, 3])

gui = mdw_gui.display_gui(v_corpus, v_corpus.documents)



from typing import Any, Dict

import pandas as pd
import penelope.utility as utility

logger = utility.getLogger('corpus_text_analysis')


def filter_by_key_value(df: pd.DataFrame, filters: Dict[str, Any] = None) -> pd.DataFrame:
    """Returns filtered dataframe based custom key/value equality filters`.

    Parameters
    ----------
    df : pd.DataFrame
        Data frame to be filtered
    filters : Dict[str, Any], optional
        [description], by default None

    Returns
    -------
    pd.DataFrame
        [description]
    """
    for k, v in (filters or {}).items():
        if k not in df.columns:
            logger.warning('Column %s does not exist in dataframe (filter_by_key_value)', k)
            continue
        df = df[df[k] == v]

    return df


def filter_document_topic_weights(
    document_topic_weights: pd.DataFrame, filters: Dict[str, Any] = None, threshold: float = 0.0
) -> pd.DataFrame:
    """Returns document's topic weights for given `year`, `topic_id`, custom `filters` and threshold.

    Parameters
    ----------
    document_topic_weights : pd.DataFrame
        Document topic weights
    filters : Dict[str, Any], optional
        [description], by default None
    threshold : float, optional
        [description], by default 0.0

    Returns
    -------
    pd.DataFrame
        [description]
    """
    df = document_topic_weights

    df = df[df.weight >= threshold]

    for k, v in (filters or {}).items():
        if k not in df.columns:
            logger.warning('Column %s does not exist in dataframe (_find_documents_for_topics)', k)
            continue
        df = df[df[k] == v]

    return df.copy()



import glob
import logging
import os
import types
import uuid

import ipywidgets as widgets
import pandas as pd
import penelope.topic_modelling as topic_modelling
import penelope.utility as utility
import penelope.vendor.gensim as gensim_utility
import penelope.vendor.textacy as textacy_utility
from IPython.display import display

from notebooks.common import TopicModelContainer

# from . topic_model_compute import compute_topic_model

logger = utility.getLogger('corpus_text_analysis')

gensim_logger = logging.getLogger('gensim')
gensim_logger.setLevel(logging.INFO)

ENGINE_OPTIONS = [
    ('MALLET LDA', 'gensim_mallet-lda'),
    ('gensim LDA', 'gensim_lda'),
    ('gensim LDA multicore', 'gensim_lda-multicore'),
    ('gensim LSI', 'gensim_lsi'),
    ('gensim HDP', 'gensim_hdp'),
    ('gensim DTM', 'gensim_dtm'),
    ('scikit LDA', 'sklearn_lda'),
    ('scikit NMF', 'sklearn_nmf'),
    ('scikit LSA', 'sklearn_lsa'),
    ('STTM   LDA', 'gensim_sttm-lda'),
    ('STTM   BTM', 'gensim_sttm-btm'),
    ('STTM   PTM', 'gensim_sttm-ptm'),
    ('STTM  SATM', 'gensim_sttm-satm'),
    ('STTM   DMM', 'gensim_sttm-dmm'),
    ('STTM  WATM', 'gensim_sttm-watm'),
]


def get_pos_options(tag_set):
    options = [
        x
        for x in tag_set.POS.unique()
        if x not in ['PUNCT', '', 'DET', 'X', 'SPACE', 'PART', 'CONJ', 'SYM', 'INTJ', 'PRON']
    ]
    return options


def get_spinner_widget(filename="images/spinner-02.gif", width=40, height=40):
    with open(filename, "rb") as image_file:
        image = image_file.read()
    return widgets.Image(value=image, format='gif', width=width, height=height, layout={'visibility': 'hidden'})


class ComputeTopicModelUserInterface:
    def __init__(self, data_folder: str, state: TopicModelContainer, documents: pd.DataFrame, **opts):
        self.terms = []
        self.data_folder = data_folder
        self.state = state
        self.documents = documents
        self.opts = opts
        self.model_widgets, self.widget_boxes = self.prepare_widgets()

    def prepare_widgets(self):

        gui = types.SimpleNamespace(
            apply_idf=widgets.ToggleButton(
                value=False,
                description='TF-IDF',
                tooltip='Apply IDF (skikit-learn) or TF-IDF (gensim)',
                icon='check',
                layout=widgets.Layout(width='115px'),
            ),
            method=widgets.Dropdown(
                description='Engine',
                options=ENGINE_OPTIONS,
                value='gensim_lda',
                layout=widgets.Layout(width='200px'),
            ),
            n_topics=widgets.IntSlider(
                description='Topics', min=2, max=100, value=20, step=1, layout=widgets.Layout(width='240px')
            ),
            max_iter=widgets.IntSlider(
                description='Iterations',
                min=100,
                max=6000,
                value=2000,
                step=10,
                layout=widgets.Layout(width='240px'),
            ),
            show_trace=widgets.ToggleButton(
                value=False,
                description='Show trace',
                disabled=False,
                icon='check',
                layout=widgets.Layout(width='115px'),
            ),
            compute=widgets.Button(
                description='Compute',
                button_style='Success',
                layout=widgets.Layout(width='115px', background_color='blue'),
            ),
            output=widgets.Output(layout={'border': '1px solid black'}),
            spinner=get_spinner_widget(),
        )

        boxes = [
            widgets.VBox(
                [
                    gui.method,
                    gui.n_topics,
                    gui.max_iter,
                ],
                layout=widgets.Layout(margin='0px 0px 0px 0px'),
            ),
            widgets.VBox(
                [
                    gui.apply_idf,
                    gui.show_trace,
                    gui.compute,
                    gui.spinner,
                ],
                layout=widgets.Layout(align_items='flex-start'),
            ),
        ]

        return gui, boxes

    def get_corpus_terms(self, corpus):
        # assert isinstance(corpus, collections.Isiterable), 'Must be a iterable!'
        return corpus

    def display(self, corpus=None):
        def buzy(is_buzy):
            self.model_widgets.compute.disabled = is_buzy
            self.model_widgets.spinner.layout.visibility = 'visible' if is_buzy else 'hidden'

        def compute_topic_model_handler(*_):

            self.model_widgets.output.clear_output()

            buzy(True)

            gensim_logger.setLevel(logging.INFO if self.model_widgets.show_trace.value else logging.WARNING)

            with self.model_widgets.output:

                try:

                    # FIXME: Generate folder name based on corpus filename and options
                    name = uuid.uuid1()

                    target_folder = os.path.join(self.data_folder, name)

                    vectorizer_args = dict(apply_idf=self.model_widgets.apply_idf.value)

                    topic_modeller_args = dict(
                        n_topics=self.model_widgets.n_topics.value,
                        max_iter=self.model_widgets.max_iter.value,
                        learning_method='online',
                        n_jobs=1,
                    )

                    method = self.model_widgets.method.value

                    train_corpus = topic_modelling.TrainingCorpus(
                        terms=list(self.get_corpus_terms(corpus)),
                        documents=self.documents,
                        vectorizer_args=vectorizer_args,
                    )

                    inferred_model = topic_modelling.infer_model(
                        train_corpus=train_corpus, method=method, engine_args=topic_modeller_args
                    )

                    inferred_topics = topic_modelling.compile_inferred_topics_data(
                        inferred_model.topic_model, train_corpus.corpus, train_corpus.id2word, train_corpus.documents
                    )

                    inferred_model.topic_model.save(os.path.join(target_folder, 'gensim.model'))
                    topic_modelling.store_model(inferred_model, target_folder)
                    inferred_topics.store(target_folder)

                    topics = topic_modelling.get_topics_unstacked(
                        self.state.topic_model,
                        n_tokens=100,
                        id2term=self.state.id2term,
                        topic_ids=self.state.relevant_topics,
                    )

                    self.state.inferred_model = inferred_model
                    self.state.inferred_topics = inferred_model
                    self.state.inferred_model = inferred_model

                    display(topics)

                except Exception as ex:
                    logger.error(ex)
                    self.state.data = None
                    raise
                finally:
                    buzy(False)

        self.model_widgets.compute.on_click(compute_topic_model_handler)

        def method_change_handler(*_):
            with self.model_widgets.output:

                self.model_widgets.compute.disabled = True
                method = self.model_widgets.method.value

                self.model_widgets.apply_idf.disabled = False
                self.model_widgets.apply_idf.description = (
                    'Apply TF-IDF' if method.startswith('gensim') else 'Apply IDF'
                )

                if 'MALLET' in method:
                    self.model_widgets.apply_idf.description = 'TF-IDF N/A'
                    self.model_widgets.apply_idf.disabled = True

                self.model_widgets.n_topics.disabled = False
                if 'HDP' in method:
                    self.model_widgets.n_topics.value = self.model_widgets.n_topics.max
                    self.model_widgets.n_topics.disabled = True

                self.model_widgets.compute.disabled = False

        self.model_widgets.method.observe(method_change_handler, 'value')

        method_change_handler()

        display(widgets.VBox([widgets.HBox(self.widget_boxes), self.model_widgets.output]))


class TextacyCorpusUserInterface(ComputeTopicModelUserInterface):
    def __init__(self, data_folder: str, state: TopicModelContainer, documents, **opts):

        super().__init__(data_folder, state, documents, **opts)

        self.substitution_filename = self.opts.get('substitution_filename', None)
        self.tagset = self.opts.get('tagset', None)

        self.corpus_widgets, self.corpus_widgets_boxes = self.prepare_textacy_widgets()
        self.widget_boxes = self.corpus_widgets_boxes + self.widget_boxes

    def display(self, corpus=None):

        # assert hasattr(corpus, 'spacy_lang), 'Must be a textaCy corpus!'

        def pos_change_handler(*_):
            with self.model_widgets.output:
                self.model_widgets.compute.disabled = True
                selected = set(self.corpus_widgets.stop_words.value)
                frequent_words = [
                    x[0]
                    for x in textacy_utility.get_most_frequent_words(
                        corpus,
                        100,
                        normalize=self.corpus_widgets.normalize.value,
                        include_pos=self.corpus_widgets.include_pos.value,
                    )
                ]
                self.corpus_widgets.stop_words.options = frequent_words
                selected = selected & set(self.corpus_widgets.stop_words.options)
                self.corpus_widgets.stop_words.value = list(selected)
                self.model_widgets.compute.disabled = False

        self.corpus_widgets.include_pos.observe(pos_change_handler, 'value')
        pos_change_handler()

        def corpus_method_change_handler(*_):
            self.corpus_widgets.ngrams.disabled = False
            if 'MALLET' in self.model_widgets.method.value:
                self.corpus_widgets.ngrams.value = [1]
                self.corpus_widgets.ngrams.disabled = True

        self.model_widgets.method.observe(corpus_method_change_handler, 'value')

        ComputeTopicModelUserInterface.display(self, corpus)

    def get_corpus_terms(self, corpus):
        pipeline = self._create_extract_pipeline(corpus=corpus)
        terms = [list(doc) for doc in pipeline.process()]
        return terms

    def _create_extract_pipeline(self, corpus):

        gui = self.corpus_widgets

        pipeline = (
            textacy_utility.ExtractPipeline.build(corpus, target=gui.normalize.value)
            .ingest(
                ngrams=gui.ngrams.value,
                as_strings=True,
                include_pos=gui.include_pos.value,
                filter_stops=gui.filter_stops.value,
                filter_punct=True,
            )
            .frequent_word_filter(max_doc_freq=gui.max_doc_freq.value)
            .infrequent_word_filter(min_freq=gui.min_freq.value)
            .remove_stopwords(extra_stopwords=set(gui.stop_words.value))
        )

        if gui.substitute_terms.value is True:
            pipeline = pipeline.substitute(subst_map=None, filename=self.substitution_filename)

        return pipeline

    def prepare_textacy_widgets(self):

        item_layout = dict(
            display='flex',
            flex_flow='row',
            justify_content='space-between',
        )

        pos_options = get_pos_options(self.tagset)

        normalize_options = {'None': False, 'Lemma': 'lemma', 'Lower': 'lower'}
        ngrams_options = {'1': [1], '1, 2': [1, 2], '1,2,3': [1, 2, 3]}
        default_include_pos = ['NOUN', 'PROPN']
        frequent_words = ['_mask_']

        gui = types.SimpleNamespace(
            min_freq=widgets.Dropdown(
                description='Min word freq',
                options=list(range(0, 11)),
                value=2,
                layout=widgets.Layout(width='200px', **item_layout),
            ),
            max_doc_freq=widgets.Dropdown(
                description='Min doc %',
                options=list(range(75, 101)),
                value=100,
                layout=widgets.Layout(width='200px', **item_layout),
            ),
            ngrams=widgets.Dropdown(
                description='n-grams', options=ngrams_options, value=[1], layout=widgets.Layout(width='200px')
            ),
            normalize=widgets.Dropdown(
                description='Normalize',
                options=normalize_options,
                value='lemma',
                layout=widgets.Layout(width='200px'),
            ),
            filter_stops=widgets.ToggleButton(
                value=True, description='Remove stopword', tooltip='Filter out stopwords', icon='check'
            ),
            substitute_terms=widgets.ToggleButton(
                value=False, description='Map words', tooltip='Substitute words', icon='check'
            ),
            include_pos=widgets.SelectMultiple(
                options=pos_options,
                value=default_include_pos,
                rows=7,
                layout=widgets.Layout(width='60px', **item_layout),
            ),
            stop_words=widgets.SelectMultiple(
                options=frequent_words, value=list([]), rows=7, layout=widgets.Layout(width='120px', **item_layout)
            ),
        )
        boxes = [
            widgets.VBox(
                [
                    gui.min_freq,
                    gui.max_doc_freq,
                    gui.normalize,
                    gui.ngrams,
                ]
            ),
            widgets.VBox(
                [gui.filter_stops, gui.substitute_terms],
                layout=widgets.Layout(margin='0px 0px 0px 10px'),
            ),
            widgets.HBox(
                [widgets.Label(value='POS', layout=widgets.Layout(width='40px')), gui.include_pos],
                layout=widgets.Layout(margin='0px 0px 0px 10px'),
            ),
            widgets.HBox(
                [widgets.Label(value='STOP'), gui.stop_words], layout=widgets.Layout(margin='0px 0px 0px 10px')
            ),
        ]
        return gui, boxes


class PreparedCorpusUserInterface(ComputeTopicModelUserInterface):
    def __init__(self, data_folder: str, state: TopicModelContainer, fn_doc_index, **opts):

        super().__init__(data_folder, state, documents=None, **opts)

        self.corpus_widgets, self.corpus_widgets_boxes = self.prepare_source_widgets()
        self.widget_boxes = self.corpus_widgets_boxes + self.widget_boxes
        self.corpus = None
        self.fn_doc_index = fn_doc_index

    def prepare_source_widgets(self):
        corpus_files = sorted(glob.glob(os.path.join(self.data_folder, '*.tokenized.zip')))
        gui = types.SimpleNamespace(
            filepath=widgets.Dropdown(
                description='Corpus', options=corpus_files, value=None, layout=widgets.Layout(width='500px')
            )
        )

        return gui, [gui.filepath]

    def get_corpus_terms(self, _):
        filepath = self.corpus_widgets.filepath.value
        self.corpus = gensim_utility.SimpleExtTextCorpus(filepath)
        doc_terms = [list(terms) for terms in self.corpus.get_texts()]
        self.documents = self.fn_doc_index(self.corpus)
        return doc_terms

    def display(self, _=None):  # pylint: disable=arguments-differ, unused-argument

        ComputeTopicModelUserInterface.display(self, None)


BUTTON_STYLE = dict(description_width='initial', button_color='lightgreen')



import types

import ipywidgets as widgets
import penelope.notebook.widgets_utils as widgets_utils
import penelope.topic_modelling as topic_modelling
import penelope.utility as utility
from IPython.display import display

import notebooks.common.topic_trends_overview_display as topic_trends_overview_display
import notebooks.political_in_newspapers.corpus_data as corpus_data
from notebooks.common import TopicModelContainer

# from beakerx import *
# from beakerx.object import beakerx
# beakerx.pandas_display_table()


def display_gui(state: TopicModelContainer):

    lw = lambda w: widgets.Layout(width=w)

    text_id = 'topic_relevance'

    publications = utility.extend(dict(corpus_data.PUBLICATION2ID), {'(ALLA)': None})
    titles = topic_modelling.get_topic_titles(state.inferred_topics.topic_token_weights, n_tokens=100)
    weighings = [(x['description'], x['key']) for x in topic_modelling.YEARLY_MEAN_COMPUTE_METHODS]

    gui = types.SimpleNamespace(
        text_id=text_id,
        text=widgets_utils.text_widget(text_id),
        flip_axis=widgets.ToggleButton(value=True, description='Flip', icon='', layout=lw("80px")),
        publication_id=widgets.Dropdown(
            description='Publication', options=publications, value=None, layout=widgets.Layout(width="200px")
        ),
        aggregate=widgets.Dropdown(
            description='Aggregate', options=weighings, value='true_mean', layout=widgets.Layout(width="250px")
        ),
        output_format=widgets.Dropdown(
            description='Output', options=['Heatmap', 'Table'], value='Heatmap', layout=lw("180px")
        ),
        output=widgets.Output(),
    )

    _current_weight_over_time = dict(publication_id=-1, weights=None)

    def weight_over_time(document_topic_weights, publication_id):
        """Cache weight over time due to the large number of ocuments"""
        if _current_weight_over_time["publication_id"] != publication_id:
            _current_weight_over_time["publication_id"] = publication_id
            df = document_topic_weights
            if publication_id is not None:
                df = df[df.publication_id == publication_id]
            _current_weight_over_time["weights"] = topic_modelling.compute_topic_yearly_means(df).fillna(0)

        return _current_weight_over_time["weights"]

    def update_handler(*_):

        gui.output.clear_output()
        gui.flip_axis.disabled = True
        gui.flip_axis.description = 'Wait!'
        with gui.output:

            weights = weight_over_time(state.inferred_topics.document_topic_weights, gui.publication_id.value)

            topic_trends_overview_display.display_heatmap(
                weights,
                titles,
                flip_axis=gui.flip_axis.value,
                aggregate=gui.aggregate.value,
                output_format=gui.output_format.value,
            )
        gui.flip_axis.disabled = False
        gui.flip_axis.description = 'Flip'

    gui.publication_id.observe(update_handler, names='value')
    gui.aggregate.observe(update_handler, names='value')
    gui.output_format.observe(update_handler, names='value')

    display(
        widgets.VBox(
            [
                widgets.HBox([gui.aggregate, gui.publication_id, gui.output_format, gui.flip_axis]),
                widgets.HBox([gui.output]),
                gui.text,
            ]
        )
    )

    update_handler()



# -*- coding: utf-8 -*-
# ---
# jupyter:
#   jupytext:
#     formats: ipynb,py:percent
#     text_representation:
#       extension: .py
#       format_name: percent
#       format_version: '1.3'
#       jupytext_version: 1.6.0
#   kernelspec:
#     display_name: Python 3
#     language: python
#     name: python3
# ---

# %% [markdown]
# https://docs.scipy.org/doc/scipy/reference/cluster.vq.html
#

# %% [markdown]
# ## Word Distribution Cluster Analysis

# %% tags=[]
# %load_ext autoreload
# %autoreload 2

# pylint: disable=wrong-import-position, import-error

import __paths__  # isort: skip

import os
import warnings

import holoviews as hv
import ipywidgets
import penelope.common.goodness_of_fit as gof
import penelope.corpus.vectorized_corpus as vectorized_corpus
import penelope.notebook.cluster_analysis_gui as cluster_analysis_gui
import penelope.notebook.word_trends.word_trend_plot_gui as word_trend_plot_gui
from bokeh.plotting import output_notebook
from IPython.display import display

root_folder = __paths__.ROOT_FOLDER
corpus_folder = os.path.join(root_folder, "output")

warnings.filterwarnings("ignore", category=FutureWarning)


output_notebook()

hv.extension("bokeh")


# %% [markdown]
# ## Load previously vectorized corpus
#
# The corpus was created using the following settings:
#  - Tokens were converted to lower case.
#  - Only tokens that contains at least one alphanumeric character (only_alphanumeric).
#  - Accents are ot removed (remove_accents)
#  - Min token length 2 (min_len)
#  - Max length not set (max_len)
#  - Numerals are removed (keep_numerals, -N)
#  - Symbols are removed (keep_symbols, -S)
#
# Use the `corpus_vectorizer` module to create a new corpus with different settings.
#
# The loaded corpus is processed in the following ways:
#
#  - Exclude tokens having a total word count less than 10000
#  - Include at most 50000 most frequent words words.
#  - Group documents by year (y_corpus).
#  - Normalize token count based on each year's total token count.
#  - Normalize token distrubution over years to 1.0 (n_corpus).
#

# %% tags=[]

y_corpus = vectorized_corpus.load_corpus(
    tag="SOU_test_L0_+N_+S",
    folder=os.path.join(root_folder, "output"),
    n_count=5000,
    n_top=50000,
    axis=1,
    keep_magnitude=False,
)

n_corpus = y_corpus.normalize(axis=0)


# %% [markdown]
# ## Deviation metrics
#
# These metrics are used to identify tokens that have a distribution that deviates the most to a uniform distribution.
# The following metrics are computed for each token:
#
# | Metric |  | |
# | :------ | :------- | :------ |
# | L2-norm | Measures distance to unform distribution. Lower bound 1/sqrt(d) is uniformity and upper bound is a 1-hot vector. |
# | Linear regression | Curve fitting to f(x) = k * x + m, where k is the slope, and m is the intercept to y axis when x is 0. A uniform curve has slope equals to 0. |
# | χ2 test | [Khan](https://www.khanacademy.org/math/statistics-probability/inference-categorical-data-chi-square-tests/chi-square-goodness-of-fit-tests/v/pearson-s-chi-square-test-goodness-of-fit) [Blog](https://towardsdatascience.com/inferential-statistic-understanding-hypothesis-testing-using-chi-square-test-eacf9fcac533) [Wikipedia](https://en.wikipedia.org/wiki/Chi-squared_test) |
# | Statistics | The min, max, mean and variance of the distribution |
# | Earth mover distance | [PDF](http://infolab.stanford.edu/pub/cstr/reports/cs/tr/99/1620/CS-TR-99-1620.ch4.pdf) [Wikipedia](https://en.wikipedia.org/wiki/Earth_mover%27s_distance)  |
# | Kullback-Leibler divergence | S = sum(pk * log(pk / qk), where pk is the token distribution and qk is the uniform distribution |
# | Entropy | Basically the same as KLD |
# | Skew | A measure of the "skewness" of the token distribution. See [scipy.stats.skew](https://docs.scipy.org/doc/scipy/reference/generated/scipy.stats.skew.html#scipy.stats.skew) |
#
# References:
#
#  - [StackExchange](stats.stackexchange.com/questions/25827/how-does-one-measure-the-non-uniformity-of-a-distribution)
#     "It just so happens, though, that the L2 norm has a simple algebraic connection to the χ2 statistic used in goodness of fit tests:
#     that's the reason it might be suitable to measure non-uniformity"
#

# %%

# pylint: disable=redefined-outer-name


def display_uniformity_metrics(x_corpus, df_gof, df_most_deviating):

    output_row = [ipywidgets.Output(), ipywidgets.Output()]
    display(ipywidgets.HBox(output_row))

    with output_row[0]:

        columns = [
            "token",
            "word_count",
            "l2_norm",
            "slope",
            "chi2_stats",
            "earth_mover",
            "kld",
            "skew",
        ]
        display(df_gof.nlargest(10000, columns=["word_count"])[columns])

    with output_row[1]:
        pass
        # columns = ['L2 token', 'L2', 'K token', 'K', 'X2 token', 'X2', 'EMD token', 'EMD', 'KLD token', 'KLD', 'Ent. token', 'Entropy']
        # df_most_deviating.columns = columns
        # display(df_most_deviating.head(100))

    gof.plot_metrics(df_gof)
    gof.plot_slopes(x_corpus, df_most_deviating, "l2_norm")

    # df_gof.to_csv('df_gof.txt', sep='\t')
    # df_most_deviating.to_csv('df_most_deviating.txt', sep='\t')


# n_corpus.data.shape[0]

df_gof = gof.compute_goddness_of_fits_to_uniform(n_corpus)
df_most_deviating = gof.compile_most_deviating_words(df_gof, n_count=10000)


display_uniformity_metrics(n_corpus, df_gof, df_most_deviating)


# %% [markdown]
# ## Word distributions

# %%

tokens = df_most_deviating["l2_norm_token"]
word_trend_plot_gui.display_gui(n_corpus, tokens)


# %% [markdown]
# ### Cluster Analysis (Hierarchical Clustering & K-means)
#

# %%

container = cluster_analysis_gui.display_gui(n_corpus, df_gof)


# %% [markdown]
#
# ### Some references
#
# |  |  |
# | :------- | :------- |
# | Overview: | https://docs.scipy.org/doc/scipy-0.19.0/reference/cluster.hierarchy.html |
# | Linkage: |https://docs.scipy.org/doc/scipy-0.19.0/reference/generated/scipy.cluster.hierarchy.linkage.html#scipy.cluster.hierarchy.linkage |
# | Reference | Daniel Mullner, “Modern hierarchical, agglomerative clustering algorithms”, [arXiv:1109.2378v1](https://arxiv.org/abs/1109.2378v1). |
# |  | [Link](https://stackabuse.com/hierarchical-clustering-with-python-and-scikit-learn/) [Link](https://github.com/giusdp/Clustering-Techniques/tree/0c78d1a893995c4603ed07216d645801ab4fdb4d)
#
# %%



# -*- coding: utf-8 -*-
# ---
# jupyter:
#   jupytext:
#     formats: ipynb,py:percent
#     text_representation:
#       extension: .py
#       format_name: percent
#       format_version: '1.3'
#       jupytext_version: 1.6.0
#   kernelspec:
#     display_name: Python 3
#     language: python
#     name: python3
# ---

# %% [markdown]
# ## Concept Context Co-Occurrences Analysis

# %% [markdown]
# ### Setup notebook
# %%
# %load_ext autoreload
# %autoreload 2
# pylint: disable=too-many-instance-attributes, unused-argument

import importlib
import warnings

import penelope.notebook.concept_co_occurrences_gui as compute_gui
import penelope.notebook.load_co_occurrences_gui as load_gui
from bokeh.plotting import output_notebook

import __paths__  # pylint: disable=unused-import
from notebooks.concept_co_occurrences.gui_callback import loaded_callback

warnings.filterwarnings("ignore", category=FutureWarning)

output_notebook()

# %% [markdown]
# ### Generate new concept context co-co_occurrences
# For long running tasks, please use the CLI `concept_co_occurrence` instead.
# This function computes new concept context co-occurrence data and stores the result in a CSV file.
# Optionally, the co-occurrence data can be transformed to a vectorized corpus to enable word trend exploration.
# %%
importlib.reload(compute_gui)
compute_gui.display_gui(data_folder=None, corpus_pattern='*sparv4.csv.zip', generated_callback=loaded_callback)

# %% [markdown]
# ### Load saved concept context co-occurrences
# %%
load_gui.display_gui(
    data_folder='/home/roger', filename_suffix="coo_concept_context.csv.zip", loaded_callback=loaded_callback
)



# Visualize topic co-occurrence
import types

import ipywidgets as widgets
import penelope.notebook.widgets_utils as widgets_utils
import penelope.topic_modelling as topic_modelling
from IPython.display import display

import notebooks.common.topic_topic_network_display as topic_topic_network_display
from notebooks.common import TopicModelContainer

# bokeh.plotting.output_notebook()


def display_gui(state: TopicModelContainer):

    lw = lambda w: widgets.Layout(width=w)
    n_topics = state.num_topics

    text_id = 'nx_topic_topic'
    layout_options = ['Circular', 'Kamada-Kawai', 'Fruchterman-Reingold']
    output_options = {'Network': 'network', 'Table': 'table', 'Excel': 'excel', 'CSV': 'csv'}
    ignore_options = [('', None)] + [('Topic #' + str(i), i) for i in range(0, n_topics)]
    year_min, year_max = state.inferred_topics.year_period

    topic_proportions = topic_modelling.compute_topic_proportions(
        state.inferred_topics.document_topic_weights, state.inferred_topics.documents.n_terms.values
    )
    titles = topic_modelling.get_topic_titles(state.inferred_topics.topic_token_weights)

    gui = types.SimpleNamespace(
        n_topics=n_topics,
        text=widgets_utils.text_widget(text_id),
        period=widgets.IntRangeSlider(
            description='Time',
            min=year_min,
            max=year_max,
            step=1,
            value=(year_min, year_min + 5),
            continues_update=False,
        ),
        scale=widgets.FloatSlider(description='Scale', min=0.0, max=1.0, step=0.01, value=0.1, continues_update=False),
        n_docs=widgets.IntSlider(description='n-docs', min=10, max=100, step=1, value=1, continues_update=False),
        threshold=widgets.FloatSlider(
            description='Threshold', min=0.01, max=1.0, step=0.01, value=0.20, continues_update=False
        ),
        output_format=widgets.Dropdown(
            description='Output', options=output_options, value='network', layout=lw('200px')
        ),
        layout=widgets.Dropdown(
            description='Layout', options=layout_options, value='Fruchterman-Reingold', layout=lw('250px')
        ),
        progress=widgets.IntProgress(min=0, max=4, step=1, value=0, layout=widgets.Layout(width="99%")),
        ignores=widgets.SelectMultiple(
            description='Ignore', options=ignore_options, value=[], rows=5, layout=lw('250px')
        ),
        node_range=widgets.IntRangeSlider(
            description='Node size', min=10, max=100, step=1, value=(20, 60), continues_update=False
        ),
        edge_range=widgets.IntRangeSlider(
            description='Edge size', min=1, max=20, step=1, value=(2, 6), continues_update=False
        ),
        output=widgets.Output(),
    )

    def tick(x=None):
        gui.progress.value = gui.progress.value + 1 if x is None else x

    def compute_handler(*_):

        gui.output.clear_output()
        tick(1)
        with gui.output:

            topic_topic_network_display.display_topic_topic_network(
                inferred_topics=state.inferred_topics,
                filters=dict(),
                period=gui.period.value,
                ignores=gui.ignores.value,
                threshold=gui.threshold.value,
                layout=gui.layout.value,
                n_docs=gui.n_docs.value,
                scale=gui.scale.value,
                node_range=gui.node_range.value,
                edge_range=gui.edge_range.value,
                output_format=gui.output_format.value,
                text_id=text_id,
                titles=titles,
                topic_proportions=topic_proportions,
            )
        tick(0)

    gui.threshold.observe(compute_handler, names='value')
    gui.n_docs.observe(compute_handler, names='value')
    gui.period.observe(compute_handler, names='value')
    gui.scale.observe(compute_handler, names='value')
    gui.node_range.observe(compute_handler, names='value')
    gui.edge_range.observe(compute_handler, names='value')
    gui.output_format.observe(compute_handler, names='value')
    gui.layout.observe(compute_handler, names='value')
    gui.ignores.observe(compute_handler, names='value')

    display(
        widgets.VBox(
            [
                widgets.HBox(
                    [
                        widgets.VBox([gui.layout, gui.threshold, gui.n_docs, gui.period]),
                        widgets.VBox([gui.ignores]),
                        widgets.VBox([gui.node_range, gui.edge_range, gui.scale]),
                        widgets.VBox([widgets.HBox([gui.output_format]), gui.progress]),
                    ]
                ),
                gui.output,
                gui.text,
            ]
        )
    )

    compute_handler()



import logging
import types
from typing import Sequence

import ipywidgets
import pandas as pd
import penelope.corpus.vectorized_corpus as vectorized_corpus
from IPython.display import display
from penelope.vendor.textacy.mdw_modified import compute_most_discriminating_terms

import notebooks.political_in_newspapers.corpus_data as corpus_data

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger("westac")


def year_range_group_indicies(
    documents: pd.DataFrame, period: Sequence[int], pub_ids: Sequence[int] = None
) -> pd.Index:
    """[summary]

    Parameters
    ----------
    documents : pd.DataFrame
        Documents meta data
    period : Sequence[int]
        Year range for group
    pub_ids : : Sequence[int], optional
        [description], by default None

    Returns
    -------
    pd.Index[int]
        [description]
    """
    assert "year" in documents.columns

    docs = documents[documents.year.between(*period)]

    if isinstance(pub_ids, int):
        pub_ids = list(pub_ids)

    if len(pub_ids or []) > 0:
        # TODO: #90 Make groupings mot generic and move to penelope
        docs = docs[(docs.publication_id.isin(pub_ids))]

    return docs.index


def load_vectorized_corpus(corpus_folder, publication_ids):
    logger.info("Loading DTM corpus...")
    bag_term_matrix, documents, id2token = corpus_data.load_as_dtm2(corpus_folder, list(publication_ids))
    token2id = {v: k for k, v in id2token.items()}
    v_corpus = vectorized_corpus.VectorizedCorpus(bag_term_matrix, token2id, documents)
    return v_corpus


id2pubs = lambda pubs: [corpus_data.PUB2ID[x] for x in pubs]


def compile_opts(gui):
    return (
        f"python mdw_runner.py "
        f"--group {','.join([corpus_data.ID2PUB[x] for x in gui.publication_ids1.value])} {gui.period1.value[0]} {gui.period1.value[1]} "
        f"--group {','.join([corpus_data.ID2PUB[x] for x in gui.publication_ids2.value])} {gui.period2.value[0]} {gui.period2.value[1]} "
        f"--top-n-terms {gui.top_n_terms.value} "
        f"--max-n-terms {gui.max_n_terms.value} "
        f"--min-df {gui.min_df.value/100.0} "
        f"--max-df {gui.max_df.value/100.0} "
    )


def display_gui(v_corpus, v_documents):

    publications = dict(corpus_data.PUBLICATION2ID)

    lw = lambda w: ipywidgets.Layout(width=w)
    year_span = (v_documents.year.min(), v_documents.year.max())
    gui = types.SimpleNamespace(
        progress=ipywidgets.IntProgress(value=0, min=0, max=5, step=1, description="", layout=lw("90%")),
        top_n_terms=ipywidgets.IntSlider(
            description="#terms",
            min=10,
            max=1000,
            value=200,
            tooltip="The total number of tokens to return for each group",
        ),
        max_n_terms=ipywidgets.IntSlider(
            description="#top",
            min=1,
            max=2000,
            value=100,
            tooltip="Only consider tokens whose DF is within the top # terms out of all terms",
        ),
        min_df=ipywidgets.FloatSlider(
            description="Min DF%",
            min=0.0,
            max=100.0,
            value=3.0,
            step=0.1,
            layout=lw("250px"),
        ),
        max_df=ipywidgets.FloatSlider(
            description="Max DF%",
            min=0.0,
            max=100.0,
            value=95.0,
            step=0.1,
            layout=lw("250px"),
        ),
        period1=ipywidgets.IntRangeSlider(
            description="Period",
            min=year_span[0],
            max=year_span[1],
            value=(v_documents.year.min(), v_documents.year.min() + 4),
            layout=lw("250px"),
        ),
        period2=ipywidgets.IntRangeSlider(
            description="Period",
            min=year_span[0],
            max=year_span[1],
            value=(v_documents.year.max() - 4, v_documents.year.max()),
            layout=lw("250px"),
        ),
        publication_ids1=ipywidgets.SelectMultiple(
            description="Publication",
            options=publications,
            rows=4,
            value=(1,),
            layout=ipywidgets.Layout(width="250px"),
        ),
        publication_ids2=ipywidgets.SelectMultiple(
            description="Publication",
            options=publications,
            rows=4,
            value=(3,),
            layout=ipywidgets.Layout(width="250px"),
        ),
        compute=ipywidgets.Button(description="Compute", icon="", button_style="Success", layout=lw("120px")),
        output=ipywidgets.Output(layout={"border": "1px solid black"}),
    )

    boxes = ipywidgets.VBox(
        [
            ipywidgets.HBox(
                [
                    ipywidgets.VBox([gui.period1, gui.publication_ids1]),
                    ipywidgets.VBox([gui.period2, gui.publication_ids2]),
                    ipywidgets.VBox(
                        [
                            gui.top_n_terms,
                            gui.max_n_terms,
                            gui.min_df,
                            gui.max_df,
                        ],
                        layout=ipywidgets.Layout(align_items="flex-end"),
                    ),
                    ipywidgets.VBox([gui.compute]),
                ]
            ),
            gui.output,
        ]
    )

    display(boxes)

    def compute_callback_handler(*_args):
        gui.output.clear_output()
        with gui.output:
            try:
                gui.compute.disabled = True

                logger.info("Using min DF %s and max DF %s", gui.min_df.value, gui.max_df.value)

                logger.info("Slicing corpus...")

                x_corpus = v_corpus.slice_by_df(
                    max_df=gui.max_df.value / 100.0,
                    min_df=gui.min_df.value / 100.0,
                    max_n_terms=gui.max_n_terms.value,
                )

                logger.info("Corpus size after DF trim %s x %s.", *x_corpus.data.shape)

                logger.info(compile_opts(gui))

                df = compute_most_discriminating_terms(
                    x_corpus,
                    top_n_terms=gui.top_n_terms.value,
                    max_n_terms=gui.max_n_terms.value,
                    group1_indices=year_range_group_indicies(
                        x_corpus.documents,
                        gui.period1.value,
                        gui.publication_ids1.value,
                    ),
                    group2_indices=year_range_group_indicies(
                        x_corpus.documents,
                        gui.period2.value,
                        gui.publication_ids2.value,
                    ),
                )
                if df is not None:
                    display(df)
                else:
                    logger.info("No data for selected groups or periods.")

            except Exception as ex:
                logger.error(ex)
            finally:
                gui.compute.disabled = False

    gui.compute.on_click(compute_callback_handler)
    return gui



# ---
# jupyter:
#   jupytext:
#     formats: ipynb,py:percent
#     text_representation:
#       extension: .py
#       format_name: percent
#       format_version: '1.3'
#       jupytext_version: 1.6.0
#   kernelspec:
#     display_name: 'Python 3.8.5 64-bit (''westac-O7wB9ikj-py3.8'': venv)'
#     metadata:
#       interpreter:
#         hash: 22b1e31d3d5f905e8a7998fc3532ca535c806f75d42474f77651a5c803dd310a
#     name: 'Python 3.8.5 64-bit (''westac-O7wB9ikj-py3.8'': venv)'
# ---

# %% tags=[]
# %%capture
# %load_ext autoreload
# %autoreload 2

# pylint: disable=wrong-import-position

import os

import pandas as pd
from penelope.corpus import TokenizedCorpus
from penelope.corpus.readers import DataFrameTextTokenizer
from penelope.corpus.tokens_transformer import TokensTransformOpts

import __paths__  # isort:skip pylint: disable=import-error, unused-import


# from notebooks.textblock_politiskt.pandas_co_occurrence import (
#     compute_co_occurrence_for_periods,
# )

root_folder = os.getcwd().split("notebooks")[0]


def create_corpus(source_filename: str, periods):

    df = pd.read_csv(source_filename, sep="\t")[["year", "txt"]]

    reader = DataFrameTextTokenizer(df, column_filters={"year": periods})

    tokens_transform_opts = TokensTransformOpts(
        to_lower=True,
        remove_accents=False,
        min_len=1,
        max_len=None,
        keep_numerals=False,
        only_alphanumeric=False,
    )
    corpus = TokenizedCorpus(reader, tokens_transform_opts=tokens_transform_opts)
    return corpus


ytw_filename = "./data/year+text_window.txt"
ytw_corpus = create_corpus(source_filename=ytw_filename, periods=1957)

# compute_co_occurrence_for_periods("./data/year+text_window.txt", 1957, target_filename="test_1957.xlsx")

# %%



# ---
# jupyter:
#   jupytext:
#     formats: ipynb,py:percent
#     text_representation:
#       extension: .py
#       format_name: percent
#       format_version: '1.3'
#       jupytext_version: 1.6.0
#   kernelspec:
#     display_name: 'Python 3.7.5 64-bit (''welfare_state_analytics'': pipenv)'
#     name: python37564bitwelfarestateanalyticspipenvb857bd21a5fc4575b483276067dc0241
# ---

# %%
# %load_ext autoreload
# %autoreload 2

import glob
import os
import sys
import zipfile

import numpy as np
import pandas as pd

root_folder = (lambda x: os.path.join(os.getcwd().split(x)[0], x))("welfare_state_analytics")
sys.path.append(root_folder)


# %% [markdown]
# ## Convert Excel data to yearly documents
# This script creates merges the text lines into a single text file for each year and news-paper.

# %%

# Convert excel to a temporary tab seperated text file

data_folder = os.path.join(root_folder, "data")
source_excel_filename = os.path.join(data_folder, "year+text_window.xlsx")
target_text_filename = os.path.join(data_folder, "year+newspaper+text.txt")
target_zip_filename = os.path.join(data_folder, "year+newspaper+text_yearly_document.txt.zip")


def create_yearly_documents(source_filename, target_name):

    df = pd.read_csv(source_filename, sep="\t")
    documents = df.fillna("").groupby(["year", "newspaper"])["txt"].apply(" ".join).reset_index()

    with zipfile.ZipFile(target_name, "w") as zf:
        for _, document in documents.iterrows():
            store_filename = "{}_{}.txt".format(document["newspaper"], document["year"])
            zf.writestr(store_filename, document["txt"], zipfile.ZIP_DEFLATED)


if not os.path.exists(target_zip_filename):
    print("Creating yearly documents...")
    # excel_to_csv(source_excel_filename, target_text_filename)
    # create_yearly_documents(target_text_filename, target_zip_filename)

print("OK!")

# %% [markdown]
# ## Run STAGGER NER tagging
# Note that archive created above must first be unzipped into a seperate folder.

# %% language="bash"
# # nohup java -Xmx4G -jar ~/source/stagger/stagger.jar -modelfile ~/source/stagger/models/swedish.bin -lang sv -tag *.txt &
#

# %% [markdown]
# ## Compile result

# %%


def read_conll_ner_tag(filename, only_ner_tags=True):

    df = pd.read_csv(filename, sep="\t", header=None, index_col=0, skip_blank_lines=True, quoting=3)
    df.columns = [
        "token",
        "lemma",
        "pos",
        "F4",
        "pos2",
        "F6",
        "F7",
        "F8",
        "F9",
        "tag",
        "type",
        "id",
    ]
    df = df[["id", "token", "pos", "tag", "type"]]

    df["parts"] = df.id.str.split("_")
    df["paper"] = df.parts.apply(lambda x: x[0] if isinstance(x, list) and len(x) > 0 else "??")
    df["year"] = df.parts.apply(lambda x: x[1].split(":")[0] if isinstance(x, list) and len(x) > 1 else "0").astype(
        np.int32
    )

    df = df[["paper", "year", "token", "tag", "type"]]

    if only_ner_tags:
        df = df.loc[df.type != "_"]

    return df


result_folder = os.path.join(data_folder, "year+newspaper+text_yearly_document")
result_files = glob.glob("{}/*.conll".format(result_folder))

df_all_tags = pd.concat([read_conll_ner_tag(filename) for filename in result_files])
df_all_tags.to_excel("year+newspaper+text_yearly_document_all_ner_tags.xlsx")


# %%



# Visualize year-to-topic network by means of topic-document-weights
import types
from typing import Sequence

import bokeh
import bokeh.plotting
import ipywidgets as widgets
import numpy as np
import penelope.network.metrics as network_metrics
import penelope.network.plot_utility as plot_utility
import penelope.network.utility as network_utility
import penelope.notebook.widgets_utils as widgets_utils
import penelope.topic_modelling as topic_modelling
import penelope.utility as utility
from IPython.display import display

import notebooks.political_in_newspapers.corpus_data as corpus_data
from notebooks.common import TopicModelContainer

TEXT_ID = 'nx_pub_topic'


# pylint: disable=too-many-locals, too-many-arguments
def plot_document_topic_network(network, layout, scale=1.0, titles=None):  # pylint: disable=unused-argument
    tools = "pan,wheel_zoom,box_zoom,reset,hover,previewsave"
    source_nodes, target_nodes = network_utility.get_bipartite_node_set(network, bipartite=0)

    source_source = network_utility.get_node_subset_source(network, layout, source_nodes)
    target_source = network_utility.get_node_subset_source(network, layout, target_nodes)
    lines_source = network_utility.get_edges_source(network, layout, scale=6.0, normalize=False)

    edges_alphas = network_metrics.compute_alpha_vector(lines_source.data['weights'])

    lines_source.add(edges_alphas, 'alphas')

    p = bokeh.plotting.figure(plot_width=1000, plot_height=600, x_axis_type=None, y_axis_type=None, tools=tools)

    _ = p.multi_line(
        xs='xs', ys='ys', line_width='weights', level='underlay', alpha='alphas', color='black', source=lines_source
    )
    _ = p.circle(x='x', y='y', size=40, source=source_source, color='lightgreen', line_width=1, alpha=1.0)

    r_topics = p.circle(x='x', y='y', size=25, source=target_source, color='skyblue', alpha=1.0)

    p.add_tools(
        bokeh.models.HoverTool(
            renderers=[r_topics],
            tooltips=None,
            callback=widgets_utils.glyph_hover_callback2(
                target_source, 'node_id', text_ids=titles.index, text=titles, element_id=TEXT_ID
            ),
        )
    )

    text_opts = dict(x='x', y='y', text='name', level='overlay', x_offset=0, y_offset=0, text_font_size='8pt')

    p.add_layout(
        bokeh.models.LabelSet(
            source=source_source, text_color='black', text_align='center', text_baseline='middle', **text_opts
        )
    )
    p.add_layout(
        bokeh.models.LabelSet(
            source=target_source, text_color='black', text_align='center', text_baseline='middle', **text_opts
        )
    )

    return p


def display_document_topic_network(
    layout_algorithm: str,
    state: TopicModelContainer,
    document_threshold: float = 0.0,
    mean_threshold: float = 0.10,
    period: Sequence[int] = None,
    ignores: Sequence[int] = None,
    scale: float = 1.0,
    aggregate: str = 'mean',
    output_format: str = 'network',
    tick=utility.noop,
):

    tick(1)

    topic_token_weights = state.inferred_topics.topic_token_weights
    document_topic_weights = state.inferred_topics.document_topic_weights

    titles = topic_modelling.get_topic_titles(topic_token_weights)

    df = document_topic_weights
    if len(period or []) == 2:
        df = df[(df.year >= period[0]) & (df.year <= period[1])]

    if len(ignores or []) > 0:
        df = df[~df.topic_id.isin(ignores)]

    df = df[(df['weight'] >= document_threshold)]

    df = df.groupby(['publication_id', 'topic_id']).agg([np.mean, np.max])['weight'].reset_index()
    df.columns = ['publication_id', 'topic_id', 'mean', 'max']

    df = df[(df[aggregate] > mean_threshold)].reset_index()

    if len(df) == 0:
        print('No data! Please change selection.')
        return

    df[aggregate] = utility.clamp_values(list(df[aggregate]), (0.1, 1.0))

    df['publication'] = df.publication_id.apply(lambda x: corpus_data.ID2PUBLICATION[x])
    df['weight'] = df[aggregate]

    network = network_utility.create_bipartite_network(
        df[['publication', 'topic_id', 'weight']], 'publication', 'topic_id'
    )
    tick()

    if output_format == 'network':
        args = plot_utility.layout_args(layout_algorithm, network, scale)
        layout = (plot_utility.layout_algorithms[layout_algorithm])(network, **args)
        tick()
        p = plot_document_topic_network(network, layout, scale=scale, titles=titles)
        bokeh.plotting.show(p)

    else:

        df = df[['publication', 'topic_id', 'weight', 'mean', 'max']]
        df.columns = ['Source', 'Target', 'weight', 'mean', 'max']
        if output_format == 'table':
            display(df)
        if output_format == 'excel':
            filename = utility.timestamp("{}_publication_topic_network.xlsx")
            df.to_excel(filename)
        if output_format == 'CSV':
            filename = utility.timestamp("{}_publication_topic_network.csv")
            df.to_csv(filename, sep='\t')

        display(df)

    tick(0)


def display_gui(state: TopicModelContainer):

    lw = lambda w: widgets.Layout(width=w)

    layout_options = ['Circular', 'Kamada-Kawai', 'Fruchterman-Reingold']
    year_min, year_max = state.inferred_topics.year_period

    n_topics = state.num_topics

    gui = types.SimpleNamespace(
        text=widgets_utils.text_widget(TEXT_ID),
        period=widgets.IntRangeSlider(
            description='Time', min=year_min, max=year_max, step=1, value=(year_min, year_max), continues_update=False
        ),
        scale=widgets.FloatSlider(description='Scale', min=0.0, max=1.0, step=0.01, value=0.1, continues_update=False),
        document_threshold=widgets.FloatSlider(
            description='Threshold(D)', min=0.0, max=1.0, step=0.01, value=0.00, continues_update=False
        ),
        mean_threshold=widgets.FloatSlider(
            description='Threshold(G)', min=0.0, max=1.0, step=0.01, value=0.10, continues_update=False
        ),
        aggregate=widgets.Dropdown(
            description='Aggregate', options=['mean', 'max'], value='mean', layout=widgets.Layout(width="200px")
        ),
        output_format=widgets.Dropdown(
            description='Output',
            options={'Network': 'network', 'Table': 'table', 'Excel': 'excel', 'CSV': 'csv'},
            value='network',
            layout=lw('200px'),
        ),
        layout=widgets.Dropdown(
            description='Layout', options=layout_options, value='Fruchterman-Reingold', layout=lw('250px')
        ),
        progress=widgets.IntProgress(min=0, max=4, step=1, value=0, layout=widgets.Layout(width="99%")),
        ignores=widgets.SelectMultiple(
            description='Ignore',
            options=[('', None)] + [('Topic #' + str(i), i) for i in range(0, n_topics)],
            value=[],
            rows=8,
            layout=lw('240px'),
        ),
    )

    def tick(x=None):
        gui.progress.value = gui.progress.value + 1 if x is None else x

    iw = widgets.interactive(
        display_document_topic_network,
        layout_algorithm=gui.layout,
        state=widgets.fixed(state),
        document_threshold=gui.document_threshold,
        mean_threshold=gui.mean_threshold,
        period=gui.period,
        ignores=gui.ignores,
        scale=gui.scale,
        aggregate=gui.aggregate,
        output_format=gui.output_format,
        tick=widgets.fixed(tick),
    )

    display(
        widgets.VBox(
            [
                widgets.HBox(
                    [
                        widgets.VBox([gui.layout, gui.document_threshold, gui.mean_threshold, gui.scale, gui.period]),
                        widgets.VBox([gui.ignores]),
                        widgets.VBox([gui.output_format, gui.progress]),
                    ]
                ),
                iw.children[-1],
                gui.text,
            ]
        )
    )
    iw.update()



import os
from dataclasses import dataclass
from typing import Dict

import ipywidgets
import pandas as pd
import penelope.common.goodness_of_fit as gof
import penelope.notebook.utility as notebook_utility
import penelope.notebook.word_trends.word_trend_plot_gui as word_trend_plot_gui
from penelope.co_occurrence.concept_co_occurrence import to_vectorized_corpus
from penelope.common.goodness_of_fit import GoodnessOfFitComputeError
from penelope.corpus import VectorizedCorpus
from penelope.notebook.ipyaggrid_utility import display_grid
from penelope.utility import getLogger

logger = getLogger()


@dataclass
class State:  # pylint: disable=too-many-instance-attributes
    concept_co_occurrences: pd.DataFrame = None
    concept_co_occurrences_metadata: Dict = None
    corpus: VectorizedCorpus = None
    corpus_folder: str = None
    corpus_tag: str = None
    compute_options: Dict = None
    goodness_of_fit: pd.DataFrame = None
    most_deviating_overview: pd.DataFrame = None
    most_deviating: pd.DataFrame = None


def update_state(
    state: State,  # pylint: disable=redefined-outer-name
    *,
    corpus: VectorizedCorpus = None,
    corpus_folder: str = None,
    corpus_tag: str = None,
    n_count: int = 10000,
    **kwargs,
):

    state.corpus = corpus
    state.corpus_folder = corpus_folder
    state.corpus_tag = corpus_tag
    state.concept_co_occurrences = kwargs.get('concept_co_occurrences', None)
    state.compute_options = kwargs.get('compute_options', None)

    if corpus is None:

        if state.concept_co_occurrences is None:
            raise ValueError("Both corpus and concept_co_occurrences cannot be None")

        state.corpus = to_vectorized_corpus(
            co_occurrences=state.concept_co_occurrences, value_column='value_n_t'
        ).group_by_year()

    state.goodness_of_fit = gof.compute_goddness_of_fits_to_uniform(state.corpus, None, verbose=False)
    state.most_deviating_overview = gof.compile_most_deviating_words(state.goodness_of_fit, n_count=n_count)
    state.most_deviating = gof.get_most_deviating_words(
        state.goodness_of_fit, 'l2_norm', n_count=n_count, ascending=False, abs_value=True
    )

    return state


def build_layout(
    state: State,  # pylint: disable=redefined-outer-name
):
    tab_trends = word_trend_plot_gui.display_gui(state.corpus, tokens=state.most_deviating, display_widgets=False)

    tab_gof = (
        notebook_utility.OutputsTabExt(["GoF", "GoF (abs)", "Plots", "Slopes"])
        .display_fx_result(0, display_grid, state.goodness_of_fit)
        .display_fx_result(1, display_grid, state.most_deviating_overview[['l2_norm_token', 'l2_norm', 'abs_l2_norm']])
        .display_fx_result(2, gof.plot_metrics, state.goodness_of_fit, plot=False, lazy=True)
        .display_fx_result(
            3, gof.plot_slopes, state.corpus, state.most_deviating, "l2_norm", 600, 600, plot=False, lazy=True
        )
    )

    layout = (
        notebook_utility.OutputsTabExt(["Data", "Explore", "Options", "GoF"])
        .display_fx_result(0, display_grid, state.concept_co_occurrences)
        .display_content(1, what=tab_trends, clear=True)
        .display_as_yaml(2, state.compute_options, clear=True, width='800px', height='600px')
        .display_content(3, tab_gof, clear=True)
    )

    return layout


state = State()


def loaded_callback(
    output: ipywidgets.Output,
    *,
    corpus: VectorizedCorpus = None,
    corpus_folder: str = None,
    corpus_tag: str = None,
    n_count: int = 10000,
    **kwargs,
):
    global state

    with output:
        try:

            output.clear_output()

            if os.environ.get('VSCODE_LOGS', None) is not None:
                logger.error("bug-check: vscode detected, aborting plot...")
                return

            _ = update_state(
                state,
                corpus=corpus,
                corpus_folder=corpus_folder,
                corpus_tag=corpus_tag,
                n_count=n_count,
                **kwargs,
            )

            build_layout(state=state).display()

        except GoodnessOfFitComputeError as ex:
            logger.info(f"Unable to compute GoF: {str(ex)}")
        except Exception as ex:
            logger.exception(ex)



import types

import ipywidgets as widgets
import penelope.notebook.widgets_utils as widgets_utils
import penelope.topic_modelling as topic_modelling
import penelope.utility as utility
from IPython.display import display

import notebooks.common.topic_trends_overview_display as topic_trends_overview_display
from notebooks.common import TopicModelContainer

# from beakerx import *
# from beakerx.object import beakerx
# beakerx.pandas_display_table()

logger = utility.get_logger()


def display_gui(state: TopicModelContainer):

    lw = lambda w: widgets.Layout(width=w)

    text_id = 'topic_relevance'

    # year_min, year_max = state.inferred_topics.year_period

    titles = topic_modelling.get_topic_titles(state.inferred_topics.topic_token_weights, n_tokens=100)
    weighings = [(x['description'], x['key']) for x in topic_modelling.YEARLY_MEAN_COMPUTE_METHODS]

    gui = types.SimpleNamespace(
        text_id=text_id,
        text=widgets_utils.text_widget(text_id),
        flip_axis=widgets.ToggleButton(value=False, description='Flip', icon='', layout=lw("80px")),
        aggregate=widgets.Dropdown(
            description='Aggregate', options=weighings, value='max_weight', layout=widgets.Layout(width="250px")
        ),
        output_format=widgets.Dropdown(
            description='Output', options=['Heatmap', 'Table'], value='Heatmap', layout=lw("180px")
        ),
        output=widgets.Output(),
    )

    def update_handler(*_):

        gui.output.clear_output()

        with gui.output:

            weights = topic_modelling.compute_topic_yearly_means(state.inferred_topics.document_topic_weights).fillna(0)

            topic_trends_overview_display.display_heatmap(
                weights,
                titles,
                flip_axis=gui.flip_axis.value,
                aggregate=gui.aggregate.value,
                output_format=gui.output_format.value,
            )

    gui.aggregate.observe(update_handler, names='value')
    gui.output_format.observe(update_handler, names='value')
    gui.flip_axis.observe(update_handler, names='value')

    display(
        widgets.VBox(
            [widgets.HBox([gui.aggregate, gui.output_format, gui.flip_axis]), widgets.HBox([gui.output]), gui.text]
        )
    )

    update_handler()



import penelope.topic_modelling as topic_modelling


class TopicModelException(Exception):
    pass


class TopicModelContainer:
    """Class for current (last) computed or loaded model"""

    _singleton = None

    def __init__(
        self,
        _inferred_data: topic_modelling.InferredModel = None,
        _inferred_topics: topic_modelling.InferredTopicsData = None,
    ):
        self._inferred_data: topic_modelling.InferredModel = None
        self._inferred_topics: topic_modelling.InferredTopicsData = None

    @staticmethod
    def singleton():
        TopicModelContainer._singleton = TopicModelContainer._singleton or TopicModelContainer()
        return TopicModelContainer._singleton

    def set_data(
        self, _inferred_data: topic_modelling.InferredModel, _inferred_topics: topic_modelling.InferredTopicsData
    ):
        """ Fix missing document attribute n_terms """
        if 'n_terms' not in _inferred_topics.documents.columns:
            assert _inferred_data.train_corpus is not None
            _inferred_topics.documents['n_terms'] = _inferred_data.train_corpus.corpus.sparse.sum(axis=0).A1

        self._inferred_data = _inferred_data
        self._inferred_topics = _inferred_topics

    @property
    def inferred_model(self) -> topic_modelling.InferredModel:
        if self._inferred_data is None:
            raise TopicModelException('Model not loaded or computed')
        return self._inferred_data

    @property
    def inferred_topics(self) -> topic_modelling.InferredTopicsData:
        return self._inferred_topics

    @property
    def topic_model(self):
        return self.inferred_model.topic_model

    @property
    def id2term(self):
        # return self.inferred_model.train_corpus.id2word
        return self.inferred_topics.id2word

    @property
    def num_topics(self):
        model = self.inferred_model.topic_model
        if model is None:
            return 0
        if hasattr(model, 'num_topics'):
            return model.num_topics
        if hasattr(model, 'n_topics'):
            return model.n_topics

        return 0

    # @property
    # def relevant_topics(self):
    #     return self._inferred_topics.relevant_topic_ids



import types
import warnings
from typing import Any, Dict

import ipywidgets as widgets
import pandas as pd
import penelope.notebook.widgets_utils as widgets_utils
import penelope.topic_modelling as topic_modelling
import penelope.utility as utility
from IPython.display import display

import notebooks.political_in_newspapers.corpus_data as corpus_data
from notebooks.common import TopicModelContainer, filter_document_topic_weights, to_text

logger = utility.get_logger()
# from beakerx import *
# from beakerx.object import beakerx
# beakerx.pandas_display_table()

warnings.filterwarnings("ignore", category=DeprecationWarning)
warnings.filterwarnings("ignore", category=FutureWarning)


def reconstitue_texts_for_topic(df: pd.DataFrame, corpus, id2token, n_top=500):

    df['text'] = df.document_id.apply(lambda x: to_text(corpus[x], id2token))
    df['pub'] = df.publication_id.apply(lambda x: corpus_data.ID2PUB[x])
    df = df.drop(['topic_id', 'year', 'publication_id'], axis=1).set_index('document_id')
    df.index.name = 'id'
    return df.sort_values('weight', ascending=False).head(n_top)


def display_texts(
    state: TopicModelContainer,
    filters: Dict[str, Any],
    threshold: float = 0.0,
    output_format: str = 'Table',
    n_top: int = 500,
):

    if state.inferred_model.train_corpus is None:
        print("Corpus is not avaliable. Please store model with corpus!")
        return

    corpus = state.inferred_model.train_corpus.corpus
    id2token = state.inferred_model.train_corpus.id2word
    document_topic_weights = state.inferred_topics.document_topic_weights

    df = filter_document_topic_weights(document_topic_weights, filters=filters, threshold=threshold)

    df = reconstitue_texts_for_topic(df, corpus, id2token, n_top=n_top)

    if len(df) == 0:
        print('No data to display for this topic and theshold')
    elif output_format == 'Table':
        display(df)


def display_gui(state: TopicModelContainer):

    year_min, year_max = state.inferred_topics.year_period
    year_options = [(x, x) for x in range(year_min, year_max + 1)]

    text_id = 'topic_document_text'
    publications = utility.extend(dict(corpus_data.PUBLICATION2ID), {'(ALLA)': None})

    gui = types.SimpleNamespace(
        n_topics=state.num_topics,
        text_id=text_id,
        text=widgets_utils.text_widget(text_id),
        year=widgets.Dropdown(
            description='Year', options=year_options, value=year_options[0][0], layout=widgets.Layout(width="200px")
        ),
        publication_id=widgets.Dropdown(
            description='Publication', options=publications, value=None, layout=widgets.Layout(width="200px")
        ),
        topic_id=widgets.IntSlider(
            description='Topic ID', min=0, max=state.num_topics - 1, step=1, value=0, continuous_update=False
        ),
        n_top=widgets.IntSlider(description='#Docs', min=5, max=500, step=1, value=75),
        threshold=widgets.FloatSlider(
            description='Threshold', min=0.0, max=1.0, step=0.01, value=0.20, continues_update=False
        ),
        output_format=widgets.Dropdown(
            description='Format', options=['Table'], value='Table', layout=widgets.Layout(width="200px")
        ),
        progress=widgets.IntProgress(min=0, max=4, step=1, value=0),
        output=widgets.Output(),
        prev_topic_id=None,
        next_topic_id=None,
    )

    gui.prev_topic_id = widgets_utils.button_with_previous_callback(gui, 'topic_id', state.num_topics)
    gui.next_topic_id = widgets_utils.button_with_next_callback(gui, 'topic_id', state.num_topics)

    def on_topic_change_update_gui(topic_id):

        if gui.n_topics != state.num_topics:
            gui.n_topics = state.num_topics
            gui.topic_id.value = 0
            gui.topic_id.max = state.num_topics - 1

        tokens = topic_modelling.get_topic_title(state.inferred_topics.topic_token_weights, topic_id, n_tokens=200)

        gui.text.value = 'ID {}: {}'.format(topic_id, tokens)

    def update_handler(*_):

        gui.output.clear_output()

        with gui.output:

            on_topic_change_update_gui(gui.topic_id.value)

            display_texts(
                state=state,
                filters=dict(year=gui.year.value, topic_id=gui.topic_id.value, publication_id=gui.publication_id.value),
                threshold=gui.threshold.value,
                n_top=gui.n_top.value,
                output_format=gui.output_format.value,
            )

    gui.topic_id.observe(update_handler, names='value')
    gui.year.observe(update_handler, names='value')
    gui.publication_id.observe(update_handler, names='value')
    gui.threshold.observe(update_handler, names='value')
    gui.n_top.observe(update_handler, names='value')
    gui.output_format.observe(update_handler, names='value')

    display(
        widgets.VBox(
            [
                widgets.HBox(
                    [
                        widgets.VBox(
                            [
                                widgets.HBox([gui.prev_topic_id, gui.next_topic_id]),
                                gui.progress,
                            ]
                        ),
                        widgets.VBox([gui.topic_id, gui.threshold, gui.n_top]),
                        widgets.VBox([gui.publication_id, gui.year]),
                        widgets.VBox([gui.output_format]),
                    ]
                ),
                gui.text,
                gui.output,
            ]
        )
    )

    update_handler()



import types
import warnings

import ipywidgets as widgets
import penelope.notebook.widgets_utils as widgets_utils
import penelope.topic_modelling as topic_modelling
import penelope.utility as utility
from IPython.display import display

import notebooks.common.topic_trend_display as topic_trend_display
import notebooks.political_in_newspapers.corpus_data as corpus_data
from notebooks.common import TopicModelContainer

warnings.filterwarnings("ignore", category=DeprecationWarning)
warnings.filterwarnings("ignore", category=FutureWarning)


def display_gui(state: TopicModelContainer, extra_filter=None):  # pylint: disable=unused-argument

    text_id = 'topic_share_plot'
    publications = utility.extend(dict(corpus_data.PUBLICATION2ID), {'(ALLA)': None})

    weighings = [(x['description'], x['key']) for x in topic_modelling.YEARLY_MEAN_COMPUTE_METHODS]

    gui = types.SimpleNamespace(
        n_topics=state.num_topics,
        text_id=text_id,
        text=widgets_utils.text_widget(text_id),
        publication_id=widgets.Dropdown(
            description='Publication', options=publications, value=None, layout=widgets.Layout(width="200px")
        ),
        aggregate=widgets.Dropdown(
            description='Aggregate', options=weighings, value='true_mean', layout=widgets.Layout(width="200px")
        ),
        normalize=widgets.ToggleButton(description='Normalize', value=True, layout=widgets.Layout(width="120px")),
        topic_id=widgets.IntSlider(
            description='Topic ID', min=0, max=state.num_topics - 1, step=1, value=0, continuous_update=False
        ),
        output_format=widgets.Dropdown(
            description='Format', options=['Chart', 'Table'], value='Chart', layout=widgets.Layout(width="200px")
        ),
        progress=widgets.IntProgress(min=0, max=4, step=1, value=0),
        output=widgets.Output(),
        prev_topic_id=None,
        next_topic_id=None,
    )

    gui.prev_topic_id = widgets_utils.button_with_previous_callback(gui, 'topic_id', state.num_topics)
    gui.next_topic_id = widgets_utils.button_with_next_callback(gui, 'topic_id', state.num_topics)

    def on_topic_change_update_gui(topic_id):

        if gui.n_topics != state.num_topics:
            gui.n_topics = state.num_topics
            gui.topic_id.value = 0
            gui.topic_id.max = state.num_topics - 1

        tokens = topic_modelling.get_topic_title(state.inferred_topics.topic_token_weights, topic_id, n_tokens=200)

        gui.text.value = 'ID {}: {}'.format(topic_id, tokens)

    _current_weight_over_time = dict(publication_id=-1, weights=None)

    def weight_over_time(document_topic_weights, publication_id):
        """Cache weight over time due to the large number of ocuments"""
        if _current_weight_over_time["publication_id"] != publication_id:
            _current_weight_over_time["publication_id"] = publication_id
            df = document_topic_weights
            if publication_id is not None:
                df = df[df.publication_id == publication_id]
            _current_weight_over_time["weights"] = topic_modelling.compute_topic_yearly_means(df).fillna(0)

        return _current_weight_over_time["weights"]

    def update_handler(*_):

        gui.output.clear_output()

        with gui.output:

            on_topic_change_update_gui(gui.topic_id.value)

            weights = weight_over_time(state.inferred_topics.document_topic_weights, gui.publication_id.value)

            topic_trend_display.display(
                weight_over_time=weights,
                topic_id=gui.topic_id.value,
                year_range=state.inferred_topics.year_period,
                aggregate=gui.aggregate.value,
                normalize=gui.normalize.value,
                output_format=gui.output_format.value,
            )

    gui.topic_id.observe(update_handler, names='value')
    gui.publication_id.observe(update_handler, names='value')
    gui.normalize.observe(update_handler, names='value')
    gui.aggregate.observe(update_handler, names='value')
    gui.output_format.observe(update_handler, names='value')

    display(
        widgets.VBox(
            [
                widgets.HBox(
                    [
                        widgets.VBox(
                            [
                                widgets.HBox([gui.prev_topic_id, gui.next_topic_id]),
                                gui.progress,
                            ]
                        ),
                        widgets.VBox([gui.topic_id]),
                        widgets.VBox([gui.publication_id]),
                        widgets.VBox([gui.aggregate, gui.output_format]),
                        widgets.VBox([gui.normalize]),
                    ]
                ),
                gui.text,
                gui.output,
            ]
        )
    )

    update_handler()



import types
import warnings

import bokeh
import bokeh.plotting
import ipywidgets as widgets
import numpy as np
import penelope.notebook.widgets_utils as widgets_utils
import penelope.topic_modelling as topic_modelling
from IPython.display import display

from notebooks.common import TopicModelContainer

warnings.filterwarnings("ignore", category=DeprecationWarning)


def plot_topic_word_distribution(tokens, **args):

    source = bokeh.models.ColumnDataSource(tokens)

    p = bokeh.plotting.figure(toolbar_location="right", **args)

    _ = p.circle(x='xs', y='ys', source=source)

    label_style = dict(level='overlay', text_font_size='8pt', angle=np.pi / 6.0)

    text_aligns = ['left', 'right']
    for i in [0, 1]:
        label_source = bokeh.models.ColumnDataSource(tokens.iloc[i::2])
        labels = bokeh.models.LabelSet(
            x='xs',
            y='ys',
            text_align=text_aligns[i],
            text='token',
            text_baseline='middle',
            y_offset=5 * (1 if i == 0 else -1),
            x_offset=5 * (1 if i == 0 else -1),
            source=label_source,
            **label_style,
        )
        p.add_layout(labels)

    p.xaxis[0].axis_label = 'Token #'
    p.yaxis[0].axis_label = 'Probability%'
    p.ygrid.grid_line_color = None
    p.xgrid.grid_line_color = None
    p.axis.axis_line_color = None
    p.axis.major_tick_line_color = None
    p.axis.major_label_text_font_size = "6pt"
    p.axis.major_label_standoff = 0
    return p


def display_topic_tokens(
    state: TopicModelContainer, topic_id: int = 0, n_words: int = 100, output_format: str = 'Chart', gui=None
):
    def tick(n=None):
        if gui is not None:
            gui.progress.value = (gui.progress.value + 1) if n is None else n

    if gui is not None and gui.n_topics != state.num_topics:
        gui.n_topics = state.num_topics
        gui.topic_id.value = 0
        gui.topic_id.max = state.num_topics - 1

    tick(1)

    tokens = (
        topic_modelling.get_topic_tokens(state.inferred_topics.topic_token_weights, topic_id=topic_id, n_tokens=n_words)
        .copy()
        .drop('topic_id', axis=1)
        .assign(weight=lambda x: 100.0 * x.weight)
        .sort_values('weight', axis=0, ascending=False)
        .reset_index()
        .head(n_words)
    )

    if len(tokens) == 0:
        print("No data! Please change selection.")
        return

    if output_format == 'Chart':
        tick()
        tokens = tokens.assign(xs=tokens.index, ys=tokens.weight)
        p = plot_topic_word_distribution(
            tokens, plot_width=1200, plot_height=500, title='', tools='box_zoom,wheel_zoom,pan,reset'
        )
        bokeh.plotting.show(p)
        tick()
    else:
        display(tokens)

    tick(0)


def display_gui(state: TopicModelContainer):

    text_id = 'wc01'
    output_options = ['Chart', 'Table']

    gui = types.SimpleNamespace(
        n_topics=state.num_topics,
        text_id=text_id,
        text=widgets_utils.text_widget(text_id),
        topic_id=widgets.IntSlider(description='Topic ID', min=0, max=state.num_topics - 1, step=1, value=0),
        n_words=widgets.IntSlider(description='#Words', min=5, max=500, step=1, value=75),
        output_format=widgets.Dropdown(
            description='Format', options=output_options, value=output_options[0], layout=widgets.Layout(width="200px")
        ),
        progress=widgets.IntProgress(min=0, max=4, step=1, value=0, layout=widgets.Layout(width="95%")),
        prev_topic_id=None,
        next_topic_id=None,
    )

    gui.prev_topic_id = widgets_utils.button_with_previous_callback(gui, 'topic_id', state.num_topics)
    gui.next_topic_id = widgets_utils.button_with_next_callback(gui, 'topic_id', state.num_topics)

    iw = widgets.interactive(
        display_topic_tokens,
        state=widgets.fixed(state),
        topic_id=gui.topic_id,
        n_words=gui.n_words,
        output_format=gui.output_format,
        gui=widgets.fixed(gui),
    )

    display(
        widgets.VBox(
            [
                gui.text,
                widgets.HBox([gui.prev_topic_id, gui.next_topic_id, gui.topic_id, gui.n_words, gui.output_format]),
                gui.progress,
                iw.children[-1],
            ]
        )
    )

    iw.update()



# Visualize year-to-topic network by means of topic-document-weights
import types

import bokeh
import ipywidgets as widgets
import penelope.network.metrics as network_metrics
import penelope.network.plot_utility as network_plot
import penelope.network.utility as network_utility
import penelope.notebook.ipyaggrid_utility as ipyaggrid_utility
import penelope.notebook.widgets_utils as widget_utils
import penelope.topic_modelling as topic_modelling
import penelope.utility as utility
from ipyaggrid import Grid
from IPython.display import display

from notebooks.common import TopicModelContainer

logger = utility.getLogger("westac")


def plot_document_topic_network(
    network, layout, scale=1.0, titles=None
):  # pylint: disable=unused-argument, too-many-locals
    tools = "pan,wheel_zoom,box_zoom,reset,hover,save"
    year_nodes, topic_nodes = network_utility.get_bipartite_node_set(network, bipartite=0)

    year_source = network_utility.get_node_subset_source(network, layout, year_nodes)
    topic_source = network_utility.get_node_subset_source(network, layout, topic_nodes)
    lines_source = network_utility.get_edges_source(network, layout, scale=6.0, normalize=False)

    edges_alphas = network_metrics.compute_alpha_vector(lines_source.data["weights"])

    lines_source.add(edges_alphas, "alphas")

    p = bokeh.plotting.figure(
        plot_width=1000,
        plot_height=600,
        x_axis_type=None,
        y_axis_type=None,
        tools=tools,
    )

    _ = p.multi_line(
        xs="xs",
        ys="ys",
        line_width="weights",
        alpha="alphas",
        level="underlay",
        color="black",
        source=lines_source,
    )
    _ = p.circle(
        x="x",
        y="y",
        size=40,
        source=year_source,
        color="lightgreen",
        line_width=1,
        alpha=1.0,
    )

    r_topics = p.circle(x="x", y="y", size=25, source=topic_source, color="skyblue", alpha=1.00)

    callback = widget_utils.glyph_hover_callback2(
        topic_source, "node_id", text_ids=titles.index, text=titles, element_id="nx_id1"
    )

    p.add_tools(bokeh.models.HoverTool(renderers=[r_topics], tooltips=None, callback=callback))

    text_opts = dict(
        x="x",
        y="y",
        text="name",
        level="overlay",
        x_offset=0,
        y_offset=0,
        text_font_size="8pt",
    )

    p.add_layout(
        bokeh.models.LabelSet(
            source=year_source, text_color="black", text_align="center", text_baseline="middle", **text_opts
        )
    )
    p.add_layout(
        bokeh.models.LabelSet(
            source=topic_source, text_color="black", text_align="center", text_baseline="middle", **text_opts
        )
    )

    return p


def display_as_grid(df):

    column_defs = [
        {"headerName": "Index", "field": "index"},
        {"headerName": "Document_Id", "field": "document_id"},
        {"headerName": "Topic_Id", "field": "topic_id"},
        {
            "headerName": "Weight",
            "field": "weight",
            "cellRenderer": "function(params) { return params.value.toFixed(6); }",
        },
        {"headerName": "Filename", "field": "filename"},
        {"headerName": "N_Raw_Tokens", "field": "n_raw_tokens"},
        {"headerName": "N_Tokens", "field": "n_tokens"},
        {"headerName": "N_Terms", "field": "n_terms"},
        {"headerName": "Year", "field": "year", "cellRenderer": None},
        {"headerName": "Title", "field": "title"},
    ]

    grid_options = dict(columnDefs=column_defs, **ipyaggrid_utility.DEFAULT_GRID_OPTIONS)
    g = Grid(grid_data=df, grid_options=grid_options, **ipyaggrid_utility.DEFAULT_GRID_STYLE)
    display(g)


def display_document_topic_network(  # pylint: disable=too-many-locals)
    layout_algorithm,
    inferred_topics: topic_modelling.InferredTopicsData,
    threshold=0.10,
    period=None,
    ignores=None,
    scale=1.0,
    output_format="network",
    tick=utility.noop,
):

    tick(1)

    topic_token_weights = inferred_topics.topic_token_weights
    document_topic_weights = inferred_topics.document_topic_weights

    titles = topic_modelling.get_topic_titles(topic_token_weights)

    df = document_topic_weights[document_topic_weights.weight > threshold].reset_index()

    if len(period or []) == 2:
        df = df[(df.year >= period[0]) & (df.year <= period[1])]

    if len(ignores or []) > 0:
        df = df[~df.topic_id.isin(ignores)]

    df["weight"] = utility.clamp_values(list(df.weight), (0.1, 2.0))

    if len(df) == 0:
        print("No data")
        return

    if "filename" not in df:
        df = df.merge(
            inferred_topics.documents["filename"],
            left_on="document_id",
            right_index=True,
        )

    df["title"] = df.filename

    network = network_utility.create_bipartite_network(df, "title", "topic_id")
    tick()

    if output_format == "network":
        if layout_algorithm == "Circular":
            args = dict(dim=2, center=None, scale=1.0)
        else:
            args = network_plot.layout_args(layout_algorithm, network, scale)
        layout = (network_plot.layout_algorithms[layout_algorithm])(network, **args)
        tick()
        p = plot_document_topic_network(network, layout, scale=scale, titles=titles)
        bokeh.plotting.show(p)

    elif output_format == "table":
        display_as_grid(df)

    tick(0)


def display_gui(state: TopicModelContainer):

    lw = lambda w: widgets.Layout(width=w)

    inferred_topics: topic_modelling.InferredTopicsData = state.inferred_topics

    text_id = "nx_id1"
    layout_options = ["Circular", "Kamada-Kawai", "Fruchterman-Reingold"]
    year_min, year_max = inferred_topics.year_period

    n_topics = len(inferred_topics.topic_ids)

    gui = types.SimpleNamespace(
        text=widget_utils.text_widget(text_id),
        period=widgets.IntRangeSlider(
            description="Time",
            min=year_min,
            max=year_max,
            step=1,
            value=(year_min, year_min + 5),
            continues_update=False,
        ),
        scale=widgets.FloatSlider(
            description="Scale",
            min=0.0,
            max=1.0,
            step=0.01,
            value=0.1,
            continues_update=False,
        ),
        threshold=widgets.FloatSlider(
            description="Threshold",
            min=0.0,
            max=1.0,
            step=0.01,
            value=0.50,
            continues_update=False,
        ),
        output_format=widgets.Dropdown(
            description="Output",
            options={"Network": "network", "Table": "table"},
            value="network",
            layout=lw("200px"),
        ),
        layout=widgets.Dropdown(
            description="Layout",
            options=layout_options,
            value="Fruchterman-Reingold",
            layout=lw("250px"),
        ),
        progress=widgets.IntProgress(min=0, max=4, step=1, value=0, layout=widgets.Layout(width="99%")),
        ignores=widgets.SelectMultiple(
            description="Ignore",
            options=[("", None)] + [("Topic #" + str(i), i) for i in range(0, n_topics)],
            value=[],
            rows=8,
            layout=lw("180px"),
        ),
        output=widgets.Output(),
    )

    def tick(x=None):
        gui.progress.value = gui.progress.value + 1 if x is None else x

    def update_handler(*_):

        gui.output.clear_output()

        with gui.output:

            display_document_topic_network(
                layout_algorithm=gui.layout.value,
                inferred_topics=inferred_topics,
                threshold=gui.threshold.value,
                period=gui.period.value,
                ignores=gui.ignores.value,
                scale=gui.scale.value,
                output_format=gui.output_format.value,
                tick=tick,
            )

    gui.threshold.observe(update_handler, names='value')
    gui.period.observe(update_handler, names='value')
    gui.scale.observe(update_handler, names='value')
    gui.output_format.observe(update_handler, names='value')
    gui.layout.observe(update_handler, names='value')
    gui.ignores.observe(update_handler, names='value')

    # FIXME: Add button

    w = widgets.VBox(
        [
            widgets.HBox(
                [
                    widgets.VBox([gui.layout, gui.threshold, gui.scale, gui.period]),
                    widgets.VBox([gui.ignores]),
                    widgets.VBox([gui.output_format, gui.progress]),
                ]
            ),
            gui.output,
            gui.text,
        ]
    )

    display(w)
    update_handler()
    return w



# ---
# jupyter:
#   jupytext:
#     formats: ipynb,py:percent
#     text_representation:
#       extension: .py
#       format_name: percent
#       format_version: '1.3'
#       jupytext_version: 1.6.0
#   kernelspec:
#     display_name: 'Python 3.7.5 64-bit (''welfare_state_analytics'': pipenv)'
#     language: python
#     name: python37564bitwelfarestateanalyticspipenvb857bd21a5fc4575b483276067dc0241
# ---

# %% [markdown]
# ### Process R data files

# %%
# %load_ext autoreload
# %autoreload 2

# pylint: disable=redefined-outer-name

import __paths__  # isort:skip pylint: disable=import-error, unused-import

import os
import warnings
import zipfile

import numpy as np
import pandas as pd

from notebooks.political_in_newspapers import corpus_data

corpus_folder = '/data/westac/textblock_politisk'

warnings.simplefilter(action="ignore", category=FutureWarning)

flatten = lambda l: [item for sublist in l for item in sublist]


# %%


def load_meta_text_blocks_as_data_frame(folder):
    """ Load censored corpus data """

    filename = os.path.join(folder, corpus_data.meta_textblocks_filename)
    df_meta = pd.read_csv(filename, compression="zip", header=0, sep=",", quotechar='"', na_filter=False)
    # df_meta = df_meta[['id', 'pred_bodytext']].drop_duplicates()
    # df_meta.columns = ["doc_id", "pred_bodytext"]
    # df_meta = df_meta.set_index("doc_id")
    return df_meta


def load_reconstructed_text_corpus(folder):
    filename = os.path.join(folder, corpus_data.reconstructed_text_corpus_file)
    if not os.path.isfile(filename):
        df_corpus = corpus_data.load_corpus_dtm_as_data_frame(folder)
        df_vocabulary = corpus_data.load_vocabulary_file_as_data_frame(folder)
        id2token = df_vocabulary["token"].to_dict()
        df_reconstructed_text_corpus = (df_corpus.groupby("document_id")).apply(
            lambda x: " ".join(flatten(x["tf"] * (x["token_id"].apply(lambda y: [id2token[y]]))))
        )
        df_reconstructed_text_corpus.to_csv(filename, compression="zip", header=0, sep=",", quotechar='"')
    else:
        df_reconstructed_text_corpus = pd.read_csv(filename, compression="zip", header=None, sep=",", quotechar='"')
        df_reconstructed_text_corpus.columns = ["document_id", "text"]
        df_reconstructed_text_corpus.set_index("document_id")

    return df_reconstructed_text_corpus


def plot_document_size_distribution(df_document):

    df_term_counts = df_document.groupby("term_count").size()

    dx = pd.DataFrame({"term_count": list(range(0, df_term_counts.index.max() + 1))}).set_index("term_count")
    df_term_counts = dx.join(df_term_counts.rename("x"), how="left").fillna(0).astype(np.int)

    ax = df_term_counts.plot.bar(figsize=(20, 10), rot=45)

    ticks = ax.xaxis.get_ticklocs()
    ticklabels = [lst.get_text() for lst in ax.xaxis.get_ticklabels()]
    ax.xaxis.set_ticks(ticks[::100])
    ax.xaxis.set_ticklabels(ticklabels[::100])

    return df_term_counts


def unique_documents_per_year_and_publication(df_document):
    df = (
        df_document.groupby(["year", "publication"])
        .agg(document_count=("doc_id", "nunique"))
        .reset_index()
        .set_index(["year", "publication"])
    )
    return df


def mean_tokens_per_year(df_document):
    df = (
        df_document.groupby(["year", "publication"])
        .agg(term_count=("term_count", "mean"))
        .reset_index()
        .set_index(["year", "publication"])
        .unstack("publication")
    )
    return df


# %% [markdown]
# ### Load DTM, document index and vocabulary
# This data is loaded from CSV files exported from R (drm1)

# %%

# df = load_meta_text_blocks_as_data_frame(corpus_folder)
# rt = load_reconstructed_text_corpus(corpus_folder)

df_corpus, df_document, df_vocabulary = corpus_data.load(corpus_folder)
id2token = df_vocabulary["token"].to_dict()

df_tf = df_corpus.groupby(["document_id"]).agg(term_count=("tf", "sum"))
df_document = df_document.merge(df_tf, how="inner", right_index=True, left_index=True)


# %%

# Load DN 68, write Excel and ZP
dn68 = df_document[(df_document.publication == 'DAGENS NYHETER') & (df_document.year == 1968)]
rt = load_reconstructed_text_corpus(corpus_folder)

dn68_text = rt.merge(dn68, how='inner', left_index=True, right_index=True)[
    ['document_id', 'year', 'date', 'term_count', 'text']
]
dn68_text.columns = ['document_id', 'year', 'date', 'term_count', 'text']
dn68_text.to_excel('dn68_text.xlsx')
# dn68_text.to_csv('dn68_text.csv', sep='\t')


with zipfile.ZipFile('dn68.zip', 'w', zipfile.ZIP_DEFLATED) as out:
    i = 0
    for index, row in dn68_text.iterrows():
        i += 1
        filename = 'dn_{}_{}_{}.txt'.format(row['date'], index, 1)
        text = row['text']
        out.writestr(filename, text, zipfile.ZIP_DEFLATED)

# %% [markdown]
# ### Document size distribution
# %%

_ = plot_document_size_distribution(df_document)

# print(df.describe())


# %% [markdown]
# ### Number of documents per year and publication

# %%


unique_yearly_docs = unique_documents_per_year_and_publication(df_document)

unique_yearly_docs.unstack("publication").plot(kind="bar", subplots=True, figsize=(20, 20), layout=(2, 2), rot=45)


# %% [markdown]
# ### Numer of tokens per year and publication

# %%


df_mean_tokens_per_year = mean_tokens_per_year(df_document)
df_mean_tokens_per_year.to_excel("mean_tokens_per_year.xlsx")
# display(df_mean_tokens_per_year)
# df_mean_tokens_per_year.plot(kind='bar', subplots=True, figsize=(25,25), layout=(2,2), rot=45);


# %% [markdown]
# ### Print data sizes

# %%

print('Corpus metrics, source "dtm1.rds", arrays drm$i, drm$j, drm$v')
print("  {} max document ID".format(df_corpus.document_id.max()))
print("  {} unique document ID".format(df_corpus.document_id.unique().shape[0]))
print("  {} max token ID".format(df_corpus.token_id.max()))
print("  {} unique token ID".format(df_corpus.token_id.unique().shape[0]))

print('Document metrics, source "dtm1.rds", arrays drm$dimnames[1]')
print("  {} max ID".format(df_document.index.max()))
print("  {} unique ID".format(df_document.index.unique().shape[0]))
print("  {} unique names".format(df_document.doc_id.unique().shape[0]))

print('Vocabulary metrics, source "dtm1.rds", arrays drm$dimnames[2]')
print("  {} max ID".format(df_vocabulary.index.max()))
print("  {} unique ID".format(df_vocabulary.index.unique().shape[0]))
print("  {} unique token".format(df_vocabulary.token.unique().shape[0]))

# df_document.groupby('doc_id').filter(lambda x: len(x) > 1).head()



import logging
import types

import ipywidgets
from IPython.display import display

logger = logging.getLogger(__name__)


def display_most_discriminating_terms(df):
    display(df)


def display_gui(x_corpus, x_documents, compute_callback, display_callback):

    lw = lambda w: ipywidgets.Layout(width=w)
    year_span = (x_documents.year.min(), x_documents.year.max())
    gui = types.SimpleNamespace(
        progress=ipywidgets.IntProgress(value=0, min=0, max=5, step=1, description='', layout=lw('90%')),
        top_n_terms=ipywidgets.IntSlider(
            description='#terms',
            min=10,
            max=1000,
            value=100,
            tooltip='The total number of most discriminating terms to return for each group',
        ),
        max_n_terms=ipywidgets.IntSlider(
            description='#top',
            min=1,
            max=2000,
            value=2000,
            tooltip='Only consider terms whose document frequency is within the top # terms out of all terms',
        ),
        period1=ipywidgets.IntRangeSlider(
            description='Period',
            min=year_span[0],
            max=year_span[1],
            value=(x_documents.year.min(), x_documents.year.min() + 4),
            layout=lw('250px'),
        ),
        period2=ipywidgets.IntRangeSlider(
            description='Period',
            min=year_span[0],
            max=year_span[1],
            value=(x_documents.year.max() - 4, x_documents.year.max()),
            layout=lw('250px'),
        ),
        compute=ipywidgets.Button(description='Compute', icon='', button_style='Success', layout=lw('120px')),
        output=ipywidgets.Output(layout={'border': '1px solid black'}),
    )

    boxes = ipywidgets.VBox(
        [
            ipywidgets.HBox(
                [
                    ipywidgets.VBox([gui.period1, gui.period2]),
                    ipywidgets.VBox(
                        [
                            gui.top_n_terms,
                            gui.max_n_terms,
                        ],
                        layout=ipywidgets.Layout(align_items='flex-end'),
                    ),
                    ipywidgets.VBox([gui.compute]),
                ]
            ),
            gui.output,
        ]
    )

    display(boxes)

    def compute_callback_handler(*_):
        gui.output.clear_output()
        with gui.output:
            try:

                gui.compute.disabled = True

                df = compute_callback(
                    x_corpus,
                    x_documents,
                    top_n_terms=gui.top_n_terms.value,
                    max_n_terms=gui.max_n_terms.value,
                    period1=gui.period1.value,
                    period2=gui.period2.value,
                )

                if df is not None:
                    display_callback(df)
                else:
                    logger.info('No data for selected groups or periods.')

            except Exception as ex:
                logger.error(ex)
            finally:
                gui.compute.disabled = False

    gui.compute.on_click(compute_callback_handler)
    return gui



import os

import pandas as pd
from penelope.co_occurrence import to_co_occurrence_matrix
from penelope.corpus.readers import dataframe_text_tokenizer

# NOTE THIS FILE COMPUTES COUCCURRENCE FROM PREDEFINED WINDOWS READ FROM EXCEL FILE!


def load_text_windows(filename: str):
    """Reads excel file "filename" and returns content as a Pandas DataFrame.
    The file is written to tsv the first time read for faster subsequent reads.

    Parameters
    ----------
    filename : str
        Name of excel file that has two columns: year and txt

    Returns
    -------
    [DataFrame]
        Content of filename as a DataFrame

    Raises
    ------
    FileNotFoundError
    """
    filepath = os.path.abspath(filename)

    if not os.path.isdir(filepath):
        raise FileNotFoundError("Path {filepath} does not exist!")

    filebase = os.path.basename(filename).split('.')[0]
    textfile = os.path.join(filepath, filebase + '.txt')

    if not os.path.isfile(textfile):
        df = pd.read_excel(filename)
        df.to_csv(textfile, sep='\t')

    df = pd.read_csv(textfile, sep='\t')[['newspaper', 'year', 'txt']]

    return df


def compute_for_period_newpaper(df, period, newspaper, min_count, options):
    reader = dataframe_text_tokenizer.DataFrameTextTokenizer(df, year=period, newspaper=newspaper)
    df_y = to_co_occurrence_matrix(reader, min_count=min_count, **options)
    df_y['newspaper'] = newspaper
    df_y['period'] = str(period)
    return df_y


def compute_co_occurrence_for_periods(source_filename, newspapers, periods, target_filename, min_count=1, **options):

    columns = ['newspaper', 'period', 'w1', 'w2', 'value', 'value_n_d', 'value_n_t']

    df = pd.read_csv(source_filename, sep='\t')[['newspaper', 'year', 'txt']]
    df_r = pd.DataFrame(columns=columns)

    n_documents = 0
    for newspaper in newspapers:
        for period in periods:
            print("Processing: {} {}...".format(newspaper, period))
            df_y = compute_for_period_newpaper(df, period, newspaper, min_count, options)
            df_r = df_r.append(df_y[columns], ignore_index=True)
            n_documents += len(df_y)

    print("Done! Processed {} rows...".format(n_documents))

    # Scale a normalized data matrix to the [0, 1] range:
    df_r['value_n_t'] = df_r.value_n_t / df_r.value_n_t.max()
    df_r['value_n_d'] = df_r.value_n_d / df_r.value_n_d.max()

    extension = target_filename.split(".")[-1]
    if extension == ".xlsx":
        df_r.to_excel(target_filename, index=False)
    elif extension in ["zip", "gzip"]:
        df_r.to_csv(target_filename, sep='\t', compression=extension, index=False, header=True)
    else:
        df_r.to_csv(target_filename, sep='\t', index=False, header=True)



# ---
# jupyter:
#   jupytext:
#     formats: ipynb,py:percent
#     text_representation:
#       extension: .py
#       format_name: percent
#       format_version: '1.3'
#       jupytext_version: 1.6.0
#   kernelspec:
#     display_name: Python 3
#     language: python
#     name: python3
# ---

# %% [markdown]
# # Prepare Riksdagens Protokoll for Text Analysis
#
# ## Download Text from KB-LAB API (JSON)
#
# Use Python script `extract_json_text.py` to download `content.json` and `meta.json`. The script downloads queries packagers having "protocoll" tag (query `{ "tags": "protokoll" }`).
#
# ```bash
# cd source/westac_data
# pipenv shell
# cd src/kb_labb
# nohup python download_protocol_content_json.py >& run.log &
# ```
#
# The result is stored as a Zip archive.
#
# ### Extract text from JSON and store as a corpus of individual text files
#
# Use the script `extract_json_text.py` to extract the text from the JSON files.
#
# ```bash
# python extract_json_text.py --source-filename ~/tmp/riksdagens_protokoll_content.zip --target-filename ~/tmp/riksdagens_protokoll_content_corpus.zip
# ```
#
# The resulting Zip file contains the text files named as `prot_yyyyyy__NN.txt`. One file per protocoll.
#
# ### Vectorize the corpus to a BoW corpus (westac.VectorizedCorpus)
#
# Use the script `vectorize_protocols.py` to create a BoW corpus.
#
# ```bash
# python vectorize_protocols.py --source-filename ~/tmp/riksdagens_protokoll_content.zip --target-filename ~/tmp/riksdagens_protokoll_content_corpus.zip
# ```
#
# The script calls `generate_corpus` in `westac.corpus.corpus_vectorizer`:
#
# ```python
# import westac.corpus.corpus_vectorizer as corpus_vectorizer
#
# kwargs = dict( ...vectorize arguments...)
# corpus_filename = ...
# output_folder = ...
#
# corpus_vectorizer.generate_corpus(corpus_filename, output_folder=output_folder, **kwargs)
#
# ```
#
# The resulting corpus are stored in the specified output folder in two files; a numpy file containing the DTM and a Pythin pickled file with the dictionary and a document index.
#
# ### Prepare text files for Sparv
#
# The Sparv pipeline requires that the individual document are stored as (individual) XML files. The shell script `sparvit-to-xml` can be used to add a root tag to all text files in a Zip archive. The resulting XML files iare stored as a new Zip archive.
#
# ```bash
#  sparvit-to-xml --input riksdagens_protokoll_content_corpus.zip --output riksdagens_protokoll_content_corpus_xml.zip
#  ```



# -*- coding: utf-8 -*-
# ---
# jupyter:
#   jupytext:
#     formats: ipynb,py:percent
#     text_representation:
#       extension: .py
#       format_name: percent
#       format_version: '1.3'
#       jupytext_version: 1.6.0
#   kernelspec:
#     display_name: Python 3
#     language: python
#     name: python3
# ---

# %% [markdown]
# ## Text Analysis - Topic Modelling
# ### <span style='color: green'>SETUP </span> Prepare and Setup Notebook <span style='float: right; color: red'>MANDATORY</span>

# %%
# %%capture

# pylint: disable=wrong-import-position
# %load_ext autoreload
# %autoreload 2

import __paths__  # isort:skip pylint: disable=import-error, unused-import

import bokeh.plotting
from IPython.core.interactiveshell import InteractiveShell
from penelope.utility import get_logger

# FIXME #95 Topic Modelling notebook setup celldisplays a load of trace information when run
import notebooks.common.load_topic_model_gui as load_gui
import notebooks.common.topic_word_distribution_gui as topic_word_distribution_gui
import notebooks.common.topic_wordcloud_gui as wordcloud_gui
import notebooks.statens_offentliga_utredningar.focus_topics_network_gui as focus_gui
import notebooks.statens_offentliga_utredningar.topic_document_network_gui as topic_document_gui
import notebooks.statens_offentliga_utredningar.topic_documents_gui as documents_gui
import notebooks.statens_offentliga_utredningar.topic_topic_network_gui as topic_topic_gui
import notebooks.statens_offentliga_utredningar.topic_trends_gui as trends_gui
import notebooks.statens_offentliga_utredningar.topic_trends_overview_gui as overview_gui
from notebooks.common import TopicModelContainer, setup_pandas

logger = get_logger()
InteractiveShell.ast_node_interactivity = "all"

bokeh.plotting.output_notebook()
setup_pandas()
current_state: TopicModelContainer = TopicModelContainer.singleton

corpus_folder = "/data/westac/sou_kb_labb"
# %% [markdown]
# ### <span style='color: green'>PREPARE</span> Load Topic Model <span style='float: right; color: red'>MANDATORY</span>

# %%
# FIXME #96 Add ipyaggrid for better table display
load_gui.display_gui(corpus_folder, current_state())
# load_gui.load_model(corpus_folder, current_state(), 'test.4days')

# %% [markdown]
# ### <span style='color: green;'>VISUALIZE</span> Display Topic's Word Distribution as a Wordcloud<span style='color: red; float: right'> TRY IT</span>

# %%

bokeh.plotting.output_notebook()
try:
    wordcloud_gui.display_gui(current_state())
except Exception as ex:
    logger.exception(ex)

# %% [markdown]
# ### <span style='color: green;'>VISUALIZE</span> Topic-Word Distribution<span style='color: red; float: right'>TRY IT</span>
#

# %%

bokeh.plotting.output_notebook()
try:
    topic_word_distribution_gui.display_gui(current_state())
    # topic_word_distribution_gui.display_topic_tokens(current_state(), topic_id=0, n_words=100, output_format='Chart')
except Exception as ex:
    logger.exception(ex)

# %% [markdown]
# ### <span style='color: green;'>VISUALIZE</span> Topic Trends over Time<span style='color: red; float: right'>RUN</span>


# %%

bokeh.plotting.output_notebook()
try:
    trends_gui.display_gui(current_state())
    # trends_gui.display_topic_trend(current_state().inferred_topics.document_topic_weights, topic_id=0, year=None, year_aggregate='mean', output_format='Table')
except Exception as ex:
    logger.exception(ex)

# %% [markdown]
# ### <span style='color: green;'>VISUALIZE</span> Topic Trends Overview<span style='color: red; float: right'>TRY IT</span>
#
# - The topic shares  displayed as a scattered heatmap plot using gradient color based on topic's weight in document.
# - [Stanford’s Termite software](http://vis.stanford.edu/papers/termite) uses a similar visualization.

# %%

bokeh.plotting.output_notebook()
try:
    overview_gui.display_gui(current_state())
except ValueError as ex:
    logger.exception(ex)

# %% [markdown]
# ### <span style='color: green;'>BROWSE</span> Browse Topic Documents<span style='color: red; float: right'>TRY IT</span>
#
# Displays a list of documents that the topic occurs in above a given threshold.

# %%

bokeh.plotting.output_notebook()
try:
    documents_gui.display_gui(current_state())
except Exception as ex:
    logger.exception(ex)

# %% [markdown]
# ### <span style='color: green;'>VISUALIZE</span> Topic-Topic Network<span style='color: red; float: right'>TRY IT</span>
#
# Computes weighted graph of topics co-occurring in the same document. Topics are defined as co-occurring in a document if they both have a weight above given threshold. The edge weights are the number of co-occurrences (binary yes or no). Node size reflects topic proportions over the entire corpus computed in accordance to LDAvis topic proportions.

# %% code_folding=[0]

bokeh.plotting.output_notebook()
try:
    topic_topic_gui.display_gui(current_state())
except Exception as ex:
    logger.exception(ex)

# %% [markdown]
# ### <span style='color: green;'>VISUALIZE</span> Document-Topic Network<span style='color: red; float: right'>TRY IT</span>
#


# %%
bokeh.plotting.output_notebook()
w = topic_document_gui.display_gui(current_state())

# %%
bokeh.plotting.output_notebook()
w = focus_gui.display_gui(current_state())



import warnings

import bokeh
import bokeh.plotting
import bokeh.transform
import penelope.notebook.widgets_utils as widgets_utils
import penelope.utility as utility
from IPython.display import display

# from beakerx import *
# from beakerx.object import beakerx
# beakerx.pandas_display_table()

warnings.filterwarnings("ignore", category=DeprecationWarning)
warnings.filterwarnings("ignore", category=FutureWarning)

logger = utility.get_logger()


def _setup_glyph_coloring(_, color_high=0.3):

    # colors = list(reversed(bokeh.palettes.Greens[9]))
    colors = [
        '#ffffff',
        '#f7fcf5',
        '#e5f5e0',
        '#c7e9c0',
        '#a1d99b',
        '#74c476',
        '#41ab5d',
        '#238b45',
        '#006d2c',
        '#00441b',
    ]
    mapper = bokeh.models.LinearColorMapper(palette=colors, low=0.0, high=color_high)
    color_transform = bokeh.transform.transform('weight', mapper)
    color_bar = bokeh.models.ColorBar(
        color_mapper=mapper,
        location=(0, 0),
        ticker=bokeh.models.BasicTicker(desired_num_ticks=len(colors)),
        formatter=bokeh.models.PrintfTickFormatter(format=" %5.2f"),
    )
    return color_transform, color_bar


def compute_int_range_categories(values):
    categories = values.unique()
    if all(map(utility.isint, categories)):
        # values = [ int(v) for v in values]
        categories = [str(x) for x in sorted([int(y) for y in categories])]
        return categories
    return sorted(list(categories))


HEATMAP_FIGOPTS = dict(title="Topic heatmap", toolbar_location="right", x_axis_location="above", plot_width=1200)


def plot_topic_relevance_by_year(
    df, xs, ys, flip_axis, titles, text_id, **figopts
):  # pylint: disable=too-many-arguments, too-many-locals

    line_height = 7
    if flip_axis is True:
        xs, ys = ys, xs
        line_height = 10

    # import holoviews as hv
    # from holoviews import opts

    # heatmap = hv.HeatMap((df[xs], df[ys], np.random.randn(100), np.random.randn(100)), vdims=['z', 'z2']).redim.range(z=(-2, 2))
    # heatmap.opts(opts.HeatMap(tools=['hover'], colorbar=True, width=325, toolbar='above'))

    # return heatmap

    x_range = compute_int_range_categories(df[xs])
    y_range = compute_int_range_categories(df[ys])

    color_high = max(df.weight.max(), 0.3)
    color_transform, color_bar = _setup_glyph_coloring(df, color_high=color_high)

    source = bokeh.models.ColumnDataSource(df)

    if x_range is not None:
        figopts['x_range'] = x_range

    if y_range is not None:
        figopts['y_range'] = y_range
        figopts['plot_height'] = max(len(y_range) * line_height, 600)

    p = bokeh.plotting.figure(**figopts)

    args = dict(x=xs, y=ys, source=source, alpha=1.0, hover_color='red')

    cr = p.rect(width=1, height=1, line_color=None, fill_color=color_transform, **args)

    p.x_range.range_padding = 0
    p.ygrid.grid_line_color = None
    p.xgrid.grid_line_color = None
    p.axis.axis_line_color = None
    p.axis.major_tick_line_color = None
    p.axis.major_label_text_font_size = "8pt"
    p.axis.major_label_standoff = 0
    p.xaxis.major_label_orientation = 1.0
    p.add_layout(color_bar, 'right')

    p.add_tools(
        bokeh.models.HoverTool(
            tooltips=None,
            callback=widgets_utils.glyph_hover_callback2(source, 'topic_id', titles.index, titles, text_id),
            renderers=[cr],
        )
    )
    return p


def display_heatmap(
    weights, titles, key='max', flip_axis=False, glyph='Circle', aggregate=None, output_format=None
):  # pylint: disable=unused-argument
    try:

        ''' Display aggregate value grouped by year  '''
        weights['weight'] = weights[aggregate]

        weights['year'] = weights.year.astype(str)
        weights['topic_id'] = weights.topic_id.astype(str)

        if len(weights) == 0:
            print("No data! Please change selection.")
            return

        if output_format.lower() == 'heatmap':

            p = plot_topic_relevance_by_year(
                weights,
                xs='year',
                ys='topic_id',
                flip_axis=flip_axis,
                titles=titles,
                text_id='topic_relevance',
                **HEATMAP_FIGOPTS,
            )

            bokeh.plotting.show(p)

        else:
            display(weights)

    except Exception as ex:
        # raise
        logger.error(ex)



# ---
# jupyter:
#   jupytext:
#     formats: ipynb,py:percent
#     text_representation:
#       extension: .py
#       format_name: percent
#       format_version: '1.3'
#       jupytext_version: 1.6.0
# ---


# %%

import itertools

import matplotlib.pyplot as plt
import numpy as np
from scipy.cluster.vq import kmeans, whiten

# %%

# Create 50 datapoints in two clusters a and b

# %matplotlib inline

features = np.array(
    [
        [1.9, 2.3],
        [1.5, 2.5],
        [0.8, 0.6],
        [0.4, 1.8],
        [0.1, 0.1],
        [0.2, 1.8],
        [2.0, 0.5],
        [0.3, 1.5],
        [1.0, 1.0],
    ]
)

whitened = whiten(features)
book = np.array((whitened[0], whitened[2]))

kmeans(whitened, book)

pts = 50
a = np.random.multivariate_normal([0, 0], [[4, 1], [1, 4]], size=pts)
b = np.random.multivariate_normal([30, 10], [[10, 2], [2, 1]], size=pts)
features = np.concatenate((a, b))

# Whiten data
whitened = whiten(features)

# Find 2 clusters in the data
codebook, distortion = kmeans(whitened, 2)

# Plot whitened data and cluster centers in red
plt.scatter(whitened[:, 0], whitened[:, 1])
plt.scatter(codebook[:, 0], codebook[:, 1], c="r")
plt.show()

# %%


def main():
    np.random.seed(1977)
    numvars, numdata = 4, 10
    data = 10 * np.random.random((numvars, numdata))  # pylint: disable=no-member
    fig = scatterplot_matrix(
        data,
        ["mpg", "disp", "drat", "wt"],
        linestyle="none",
        marker="o",
        color="black",
        mfc="none",
    )
    fig.suptitle("Simple Scatterplot Matrix")
    plt.show()


def scatterplot_matrix(data, names, **kwargs):
    """Plots a scatterplot matrix of subplots.  Each row of "data" is plotted
    against other rows, resulting in a nrows by nrows grid of subplots with the
    diagonal subplots labeled with "names".  Additional keyword arguments are
    passed on to matplotlib's "plot" command. Returns the matplotlib figure
    object containg the subplot grid."""
    numvars, _ = data.shape
    fig, axes = plt.subplots(nrows=numvars, ncols=numvars, figsize=(8, 8))
    fig.subplots_adjust(hspace=0.05, wspace=0.05)

    for ax in axes.flat:
        # Hide all ticks and labels
        ax.xaxis.set_visible(False)
        ax.yaxis.set_visible(False)

        # Set up ticks only on one side for the "edge" subplots...
        if ax.is_first_col():
            ax.yaxis.set_ticks_position("left")
        if ax.is_last_col():
            ax.yaxis.set_ticks_position("right")
        if ax.is_first_row():
            ax.xaxis.set_ticks_position("top")
        if ax.is_last_row():
            ax.xaxis.set_ticks_position("bottom")

    # Plot the data.
    for i, j in zip(*np.triu_indices_from(axes, k=1)):
        for x, y in [(i, j), (j, i)]:
            axes[x, y].plot(data[x], data[y], **kwargs)

    # Label the diagonal subplots...
    for i, label in enumerate(names):
        axes[i, i].annotate(label, (0.5, 0.5), xycoords="axes fraction", ha="center", va="center")

    # Turn on the proper x or y axes ticks.
    for i, j in zip(range(numvars), itertools.cycle((-1, 0))):
        axes[j, i].xaxis.set_visible(True)
        axes[i, j].yaxis.set_visible(True)

    return fig


main()



import os

import gensim
import numpy as np
import pandas as pd


def compile_mallet_document_topics(model, minimum_probability=0.001):
    def document_topics_iter(model, minimum_probability=0.0):
        data_iter = enumerate(model.load_document_topics())
        for document_id, topic_weights in data_iter:
            for (topic_id, weight) in (
                (topic_id, weight) for (topic_id, weight) in topic_weights if weight >= minimum_probability
            ):
                yield (document_id, topic_id, weight)

    data = document_topics_iter(model, minimum_probability)
    df_doc_topics = pd.DataFrame(data, columns=['document_id', 'topic_id', 'weight'])
    df_doc_topics['document_id'] = df_doc_topics.document_id.astype(np.uint32)
    df_doc_topics['topic_id'] = df_doc_topics.topic_id.astype(np.uint16)

    return df_doc_topics


def fix_int32_overflow(data_folder, model_name):

    target_folder = os.path.join(data_folder, model_name)

    model_filename = os.path.join(target_folder, 'model', 'gensim.model')

    ldaMallet = gensim.models.wrappers.LdaMallet.load(model_filename)
    ldaMallet.prefix = '{}/{}/'.format(data_folder, model_name)

    documents = pd.read_csv(os.path.join(target_folder, 'documents.zip'), '\t', header=0, index_col=0, na_filter=False)
    document_topic_weights = compile_mallet_document_topics(ldaMallet, minimum_probability=0.001)

    document_topic_weights = document_topic_weights.merge(
        documents[['publication_id', 'year']], how='inner', left_on='document_id', right_index=True
    )

    target_file = os.path.join(target_folder, 'document_topic_weights.zip')
    document_topic_weights.to_csv(target_file, '\t')
    print(' Stored new version of {}.'.format(target_file))


def run():
    corpus_folder = '/home/roger/source/welfare_state_analytics/data/textblock_politisk'
    for n_topics in [50, 100, 200, 400]:
        model_name = 'gensim_mallet-lda.topics.{}.AB.DN'.format(n_topics)
        print('Fixing {}...'.format(model_name))
        fix_int32_overflow(corpus_folder, model_name)

    print('Done!')


if __name__ == "__main__":
    run()



# Visualize topic co-occurrence
import bokeh
import bokeh.plotting
import penelope.network.plot_utility as plot_utility
import penelope.network.utility as network_utility
import penelope.utility as utility
from IPython.display import display

from notebooks.common import filter_document_topic_weights

logger = utility.get_logger()


def get_topic_titles(topic_token_weights, topic_id=None, n_words=100):
    df_temp = (
        topic_token_weights if topic_id is None else topic_token_weights[(topic_token_weights.topic_id == topic_id)]
    )
    df = (
        df_temp.sort_values('weight', ascending=False)
        .groupby('topic_id')
        .apply(lambda x: ' '.join(x.token[:n_words].str.title()))
    )
    return df


# pylint: disable=too-many-arguments, too-many-locals
def display_topic_topic_network(
    inferred_topics,
    filters,
    period=None,
    ignores=None,
    threshold=0.10,
    layout='Fruchterman-Reingold',
    n_docs=1,
    scale=1.0,
    output_format='table',
    text_id='',
    titles=None,
    topic_proportions=None,
    node_range=(20, 60),
    edge_range=(1, 10),
):
    try:

        documents = inferred_topics.documents

        if 'document_id' not in documents.columns:
            documents['document_id'] = documents.index

        df = filter_document_topic_weights(inferred_topics.document_topic_weights, filters=filters, threshold=threshold)

        if ignores is not None:
            df = df[~df.topic_id.isin(ignores)]

        if len(period or []) == 2:
            df = df[(df.year >= period[0]) & (df.year <= period[1])]

        if isinstance(period, int):
            df = df[df.year == period]

        df = df.merge(df, how='inner', left_on='document_id', right_on='document_id')
        df = df[(df.topic_id_x < df.topic_id_y)]

        df = df.groupby([df.topic_id_x, df.topic_id_y]).size().reset_index()

        df.columns = ['source', 'target', 'n_docs']

        if n_docs > 1:
            df = df[df.n_docs >= n_docs]

        if len(df) == 0:
            print('No data. Please change selections.')
            return

        if output_format == 'network':
            network = network_utility.create_network(df, source_field='source', target_field='target', weight='n_docs')
            p = plot_utility.plot_network(
                network=network,
                layout_algorithm=layout,
                scale=scale,
                threshold=0.0,
                node_description=titles,
                node_proportions=topic_proportions,
                weight_scale=1.0,
                normalize_weights=False,
                element_id=text_id,
                figsize=(1200, 800),
                node_range=node_range,
                edge_range=edge_range,
            )
            bokeh.plotting.show(p)
        else:
            df.columns = ['Source', 'Target', 'DocCount']
            if output_format == 'table':
                display(df)
            if output_format == 'excel':
                filename = utility.timestamp("{}_topic_topic_network.xlsx")
                df.to_excel(filename)
                print('Data stored in file {}'.format(filename))
            if output_format == 'csv':
                filename = utility.timestamp("{}_topic_topic_network.csv")
                df.to_csv(filename, sep='\t')
                print('Data stored in file {}'.format(filename))

    except:  # pylint: disable=bare-except
        print("No data: please adjust filters")
        # raise



# ---
# jupyter:
#   jupytext:
#     formats: ipynb,py:percent
#     text_representation:
#       extension: .py
#       format_name: percent
#       format_version: '1.3'
#       jupytext_version: 1.6.0
#   kernelspec:
#     display_name: Python 3
#     language: python
#     name: python3
# ---

# %% [markdown]
# ## Most Discriminating Terms

# %%
# %load_ext autoreload
# %autoreload 2

# pylint: disable=wrong-import-position

import os
import sys

import penelope.corpus.vectorized_corpus as vectorized_corpus
from penelope.common.most_discriminating_terms import compute_most_discriminating_terms

from notebooks.most_discriminating_words.most_discriminating_terms_gui import (
    display_gui,
    display_most_discriminating_terms,
)

root_folder = os.path.join(os.getcwd().split("welfare_state_analytics")[0], "welfare_state_analytics")

sys.path = list(set(sys.path + [root_folder]))


corpus_folder = os.path.join(root_folder, "output")

# %%
v_corpus = (
    vectorized_corpus.VectorizedCorpus.load(tag="SOU_1945-1989_NN+VB+JJ_lemma_L0_+N_+S", folder=corpus_folder)
    .slice_by_n_count(10)
    .slice_by_n_top(500000)
)


# %%

display_gui(
    v_corpus,
    v_corpus.documents,
    compute_callback=compute_most_discriminating_terms,
    display_callback=display_most_discriminating_terms,
)



import types

import ipywidgets as widgets
import pandas as pd
import penelope.notebook.widgets_utils as widgets_utils
import penelope.plot as plot_utility
import penelope.topic_modelling as topic_modelling
import penelope.utility as utility
from IPython.display import display

from notebooks.common import TopicModelContainer

pd.set_option("display.max_rows", None)
pd.set_option('max_colwidth', 200)

opts = {'max_font_size': 100, 'background_color': 'white', 'width': 900, 'height': 600}


def display_wordcloud(state: TopicModelContainer, topic_id=0, n_words=100, output_format='Wordcloud', gui=None):
    def tick(n=None):
        gui.progress.value = (gui.progress.value + 1) if n is None else n

    if gui.n_topics != state.num_topics:
        gui.n_topics = state.num_topics
        gui.topic_id.value = 0
        gui.topic_id.max = state.num_topics - 1

    tick(1)

    try:
        topic_token_weights = state.inferred_topics.topic_token_weights

        df = topic_token_weights.loc[(topic_token_weights.topic_id == topic_id)]

        tokens = topic_modelling.get_topic_title(topic_token_weights, topic_id, n_tokens=n_words)

        if len(tokens) == 0:
            print("No data! Please change selection.")
            return

        gui.text.value = 'ID {}: {}'.format(topic_id, tokens)

        tick()

        if output_format == 'Wordcloud':
            plot_utility.plot_wordcloud(df, token='token', weight='weight', max_words=n_words, **opts)
        else:
            df = topic_modelling.get_topic_tokens(topic_token_weights, topic_id=topic_id, n_tokens=n_words)
            if output_format == 'Table':
                display(df)
            if output_format == 'Excel':
                filename = utility.timestamp("{}_wordcloud_tokens.xlsx")
                df.to_excel(filename)
            if output_format == 'CSV':
                filename = utility.timestamp("{}_wordcloud_tokens.csv")
                df.to_csv(filename, sep='\t')
    except IndexError:
        print('No data for topic')
    tick(0)


def display_gui(state: TopicModelContainer):

    output_options = ['Wordcloud', 'Table', 'CSV', 'Excel']
    text_id = 'tx02'

    gui = types.SimpleNamespace(
        n_topics=state.num_topics,
        text_id=text_id,
        text=widgets.HTML(value=f"<span class='{text_id}'></span>", placeholder='', description=''),
        topic_id=widgets.IntSlider(
            description='Topic ID', min=0, max=state.num_topics - 1, step=1, value=0, continuous_update=False
        ),
        word_count=widgets.IntSlider(description='#Words', min=5, max=250, step=1, value=25, continuous_update=False),
        output_format=widgets.Dropdown(
            description='Format', options=output_options, value=output_options[0], layout=widgets.Layout(width="200px")
        ),
        progress=widgets.IntProgress(min=0, max=4, step=1, value=0, layout=widgets.Layout(width="95%")),
        prev_topic_id=None,
        next_topic_id=None,
    )

    gui.prev_topic_id = widgets_utils.button_with_previous_callback(gui, 'topic_id', state.num_topics)
    gui.next_topic_id = widgets_utils.button_with_next_callback(gui, 'topic_id', state.num_topics)

    iw = widgets.interactive(
        display_wordcloud,
        state=widgets.fixed(state),
        topic_id=gui.topic_id,
        n_words=gui.word_count,
        output_format=gui.output_format,
        gui=widgets.fixed(gui),
    )

    display(
        widgets.VBox(
            [
                gui.text,
                widgets.HBox([gui.prev_topic_id, gui.next_topic_id, gui.topic_id, gui.word_count, gui.output_format]),
                gui.progress,
                iw.children[-1],
            ]
        )
    )

    iw.update()



import types
import warnings
from os.path import join as jj
from typing import Any, Dict, List

import ipywidgets as widgets
import numpy as np
import penelope.topic_modelling as topic_modelling
import penelope.utility as utility
from ipyaggrid.grid import Grid
from IPython.display import display
from penelope.notebook.ipyaggrid_utility import DEFAULT_GRID_OPTIONS, DEFAULT_GRID_STYLE

from notebooks.common.model_container import TopicModelContainer
from notebooks.political_in_newspapers.corpus_data import extend_with_document_info

warnings.filterwarnings("ignore", category=DeprecationWarning)
warnings.filterwarnings("ignore", category=FutureWarning)

logger = utility.get_logger()


def display_as_grid(df):

    column_defs = [
        {"headerName": "Topic", "field": "topic_id"},
        {"headerName": "Tokens", "field": "tokens"},
        {
            "headerName": "alpha",
            "field": "alpha",
            "cellRenderer": "function(params) { return params.value.toFixed(6); }",
        },
    ]

    grid_options = dict(columnDefs=column_defs, **DEFAULT_GRID_OPTIONS)
    g = Grid(grid_data=df, grid_options=grid_options, **DEFAULT_GRID_STYLE)
    display(g)


# FIXME: #94 Column 'year' is missing in `documents` in model metadata (InferredTopicsData)
def temporary_bug_fixupdate_documents(inferred_topics):

    logger.info("applying temporary bug fix of missing year in documents...done!")
    documents = inferred_topics.documents
    document_topic_weights = inferred_topics.document_topic_weights

    if "year" not in documents.columns:
        documents["year"] = documents.filename.str.split("_").apply(lambda x: x[1]).astype(np.int)

    if "year" not in document_topic_weights.columns:
        document_topic_weights = extend_with_document_info(document_topic_weights, documents)

    inferred_topics.documents = documents
    inferred_topics.document_topic_weights = document_topic_weights

    assert "year" in inferred_topics.documents.columns
    assert "year" in inferred_topics.document_topic_weights.columns

    return inferred_topics


def load_model(
    corpus_folder: str,
    state: TopicModelContainer,
    model_name: str,
    model_infos: List[Dict[str, Any]] = None,
):

    model_infos = model_infos or topic_modelling.find_models(corpus_folder)
    model_info = next(x for x in model_infos if x["name"] == model_name)

    inferred_model = topic_modelling.load_model(model_info["folder"])
    inferred_topics = topic_modelling.InferredTopicsData.load(jj(corpus_folder, model_info["name"]))

    inferred_topics = temporary_bug_fixupdate_documents(inferred_topics)

    state.set_data(inferred_model, inferred_topics)

    topics = inferred_topics.topic_token_overview
    # topics.style.set_properties(**{'text-align': 'left'}).set_table_styles(
    #     [dict(selector='td', props=[('text-align', 'left')])]
    # )

    display_as_grid(topics)


def display_gui(corpus_folder: str, state: TopicModelContainer):

    model_infos = topic_modelling.find_models(corpus_folder)
    model_names = list(x["name"] for x in model_infos)

    gui = types.SimpleNamespace(
        model_name=widgets.Dropdown(description="Model", options=model_names, layout=widgets.Layout(width="40%")),
        load=widgets.Button(
            description="Load",
            button_style="Success",
            layout=widgets.Layout(width="80px"),
        ),
        output=widgets.Output(),
    )

    def load_handler(*_):  # pylint: disable=unused-argument
        gui.output.clear_output()
        try:
            gui.load.disabled = True
            with gui.output:
                if gui.model_name.value is None:
                    print("Please specify which model to load.")
                    return
                load_model(corpus_folder, state, gui.model_name.value, model_infos)
        finally:
            gui.load.disabled = False

    gui.load.on_click(load_handler)

    display(widgets.VBox([widgets.HBox([gui.model_name, gui.load]), widgets.VBox([gui.output])]))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['notebooks',
 'notebooks.bin',
 'notebooks.common',
 'notebooks.concept_co_occurrences',
 'notebooks.most_discriminating_words',
 'notebooks.named_entity_recognition',
 'notebooks.political_in_newspapers',
 'notebooks.political_in_newspapers.notebook_gui',
 'notebooks.political_in_newspapers.scripts',
 'notebooks.scrachpads',
 'notebooks.statens_offentliga_utredningar',
 'notebooks.textblock_politiskt',
 'notebooks.tfidf_word_siginificance',
 'notebooks.word_distribution_trends',
 'notebooks.word_trends']

package_data = \
{'': ['*']}

install_requires = \
['alive-progress>=1.6.1,<2.0.0',
 'appnope',
 'attrs',
 'beakerx==1.4.1',
 'bokeh',
 'click',
 'colorcet',
 'cython',
 'cytoolz',
 'decorator',
 'ftfy',
 'gensim',
 'holoviews',
 'humanfriendly',
 'humlab-penelope>=0.2.28,<0.3.0',
 'ipyaggrid',
 'ipycytoscape',
 'ipyfilechooser>=0.4.0,<0.5.0',
 'ipykernel',
 'ipympl',
 'ipywidgets',
 'joblib',
 'jupyter-bokeh',
 'jupyterlab',
 'lxml',
 'markdown',
 'matplotlib',
 'memoization',
 'more-itertools',
 'mypy>=0.782.0,<1.0.0',
 'networkx',
 'nltk',
 'numba',
 'numpy',
 'openpyxl',
 'pandas',
 'pandas-bokeh',
 'py4j',
 'pydotplus',
 'pyemd',
 'python-dotenv',
 'python-louvain',
 'qgrid>=1.3.1,<2.0.0',
 'scattertext',
 'scikit-learn',
 'scipy',
 'seaborn',
 'sklearn',
 'spacy==2.3.2',
 'statsmodels',
 'textacy==0.10.1',
 'tqdm',
 'traitlets==4.3.3',
 'wordcloud',
 'xlrd',
 'xmltodict']

setup_kwargs = {
    'name': 'humlab-westac',
    'version': '0.2.2',
    'description': 'Welfare State Analytics',
    'long_description': '# The Welfare State Analytics Text Analysis Repository\n\n## About the Project\n\nWelfare State Analytics. Text Mining and Modeling Swedish Politics, Media & Culture, 1945-1989 (WeStAc) is a digital humanities research project with five co-operatings partners: Umeå University, Uppsala University, Aalto University (Finland) and the National Library of Sweden.\n\nThe project will digitise literature, curate already digitised collections, and perform research via probabilistic methods and text mining models. WeStAc will both digitise and curate three massive textual datasets—in all, Big Data of almost four billion tokens—from the domains of Swedish politics, news media and literary culture during the second half of the 20th century.\n\n## Installation\n\n### Local install using pipenv\n\nSee [this page](https://github.com/humlab/welfare_state_analytics/wiki/How-to:-Install-notebooks-on-local-machine).\n\n### JupyterHub installation\n\nThe `westac_hub` repository contains a ready-to-use Docker setup (`Dockerfile` and `docker-compose.yml`) for a Jupyter Hub using `nginx` as reverse-proxy. The default setup uses `DockerSpawner` that spawns containers as specified in `westac_lab`, and Github for autorization (OAuth2). See the Makefile on how to build the project.\n\n### Single Docker container\n\nYou can also run the `westac_lab` container as a single Docker container if you have Docker installed on your computer.\n',
    'author': 'Roger Mähler',
    'author_email': 'roger.mahler@hotmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://westac.se',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '==3.8.5',
}


setup(**setup_kwargs)
